/*
 * changraph.h
 * Copyright (C) Petr Klapetek 2007 <pklapetek@cmi.cz>
 * 
 * changraph.h is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * changraph.h is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with callbacks.h.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
//#include <unistd.h>
#include <stdlib.h>
//#include <utime.h>
#include "changraph.h"
#include <app/gwyapp.h>
#include <libgwymodule/gwymodule-file.h>
#include <libprocess/stats.h>
#include <libgwymodule/gwymodule.h>
#include <app/gwymoduleutils.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <libprocess/level.h>
#include <string.h>
#include <glib.h>
//#include <gthread.h>

#include <app/gwyapp.h>

#include "data.h"

#ifdef G_OS_WIN32
#include <windows.h>
#endif

#define GWS_CHANGRAPH_TYPE_NAME "GwsChangraph"

static void   gws_changraph_class_init        (GwsChangraphClass *klass);
static void   gws_changraph_init              (GwsChangraph *chan);

enum {
    SELECTED,
    LAST_SIGNAL
};
static guint changraph_signals[LAST_SIGNAL] = { 0 };


static GObjectClass *parent_class = NULL;

GType
gws_changraph_get_type(void)
{
    static GType gws_changraph_type = 0;

    if (!gws_changraph_type) {
        static const GTypeInfo gws_changraph_info = {
            sizeof(GwsChangraphClass),
            NULL,
            NULL,
            (GClassInitFunc)gws_changraph_class_init,
            NULL,
            NULL,
            sizeof(GwsChangraph),
            0,
            (GInstanceInitFunc)gws_changraph_init,
            NULL,
        };

        gws_changraph_type
            = g_type_register_static(G_TYPE_OBJECT,
                                     GWS_CHANGRAPH_TYPE_NAME,
                                     &gws_changraph_info,
                                     0);
    }

    return gws_changraph_type;
}

static void
gws_changraph_class_init(GwsChangraphClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    gwy_debug("");

    parent_class = g_type_class_peek_parent(klass);

    changraph_signals[SELECTED]
	= g_signal_new("selected",
		           G_OBJECT_CLASS_TYPE(gobject_class),
		           G_SIGNAL_ACTION | G_SIGNAL_RUN_FIRST,
		           G_STRUCT_OFFSET(GwsChangraphClass, selected),
		           NULL, NULL,
		           g_cclosure_marshal_VOID__POINTER,
		           G_TYPE_NONE, 1, G_TYPE_POINTER);
}


static void
gws_changraph_init(GwsChangraph *chan)
{
    
}


void gws_changraph_change_state(GwsChangraph* chan, ChanGraphState state)
{
    chan->state = state;
    if (chan->state == CHGR_STATE_PROFILES) {
        gtk_widget_set_sensitive(chan->timeout_spin, FALSE);
        gtk_widget_set_sensitive(chan->level, TRUE);
    } else {
        gtk_widget_set_sensitive(chan->timeout_spin, TRUE);
        gtk_widget_set_sensitive(chan->level, FALSE);
    }
}


static void 
mode_changed_cb(GtkComboBox *combo, GwsChangraph* chan)
{
   /*copy data*/
   if (chan->running) chan->channel = gtk_combo_box_get_active(combo);

}
static void 
level_changed_cb(GtkComboBox *combo, GwsChangraph* chan)
{
   /*copy data*/
   if (chan->running) chan->leveling = gtk_combo_box_get_active(combo);

}

static void 
fit_linear(gdouble *xdata, gdouble *ydata, gint ndata) 
{
    gdouble sumxi, sumxixi, av, bv;
    gdouble sumsixi = 0.0;
    gdouble sumsi = 0.0;
    gdouble n = ndata;
    gint i;

    sumxi = (n-1)/2.0;
    sumxixi = (2*n-1)*(n-1)/6.0;

    for (i = 0; i < ndata; i++) {
        sumsi += ydata[i];
        sumsixi += ydata[i] * i;
    }
    sumsi /= n;
    sumsixi /= n;

    bv = (sumsixi - sumsi*sumxi)/(sumxixi - sumxi*sumxi);
    av = (sumsi*sumxixi - sumxi*sumsixi)/(sumxixi - sumxi*sumxi);

    for (i=0; i<ndata; i++) 
    {
        ydata[i] -= av + bv*i;
    } 
  
}

gpointer
chanloop(gpointer data)
{
    GwsChangraph *chan = (GwsChangraph *)data;
    gint i, actual;
    gdouble val, *ddata;
    MiniData *md = (MiniData *)chan->md;
    GwyDataField *dfield;

    while (1) {
        if (md->closingapp) 
            return FALSE;

        if (chan->running && chan->state==CHGR_STATE_FREERUNNING) {
    
           for (i=0; i<MAXGDATA; i++) {
               chan->xdatabuf[i] = i;
               get_actual_whatever(md, &val, chan->channel); 
               chan->ydatabuf[i] = val;

               usleep(chan->timeout);
               //sleep((gint)(chan->timeout*10e-3));
           }

           g_mutex_lock(chan->graph_mutex);
           for (i=0; i<MAXGDATA; i++) {
                  chan->xdata[i] = chan->xdatabuf[i];
                  chan->ydata[i] = chan->ydatabuf[i];
           }
           chan->ndata = MAXGDATA;
           chan->rndata = 0;
           g_mutex_unlock(chan->graph_mutex);

        } else if (chan->running && chan->state==CHGR_STATE_PROFILES) {

           pthread_mutex_lock(md->dfield_mutex);
           g_mutex_lock(chan->graph_mutex);
	   dfield = md->dfield[chan->channel];
           chan->ndata = gwy_data_field_get_xres(dfield); 
           g_mutex_unlock(chan->graph_mutex);

	   actual = md->actual_profile;

           //ddata = gwy_data_field_get_data(dfield);
           ddata = (gdouble *)gwy_data_field_get_data_const(dfield);
           for (i=0; i<chan->ndata; i++) {
                  chan->xdatabuf[i] = i;
                  chan->ydatabuf[i] = ddata[i + chan->ndata*actual];
           }
           pthread_mutex_unlock(md->dfield_mutex);

           g_mutex_lock(chan->graph_mutex);  //use of the extra buffer is probably not necessary
           for (i=0; i<chan->ndata; i++) {
                  chan->xdata[i] = chan->xdatabuf[i];
                  chan->ydata[i] = chan->ydatabuf[i];
           }
           chan->rndata = 0;
           if (md->c_sc_reverse) {
           }

           if (chan->leveling) fit_linear(chan->xdata, chan->ydata, chan->ndata);
           g_mutex_unlock(chan->graph_mutex);

           usleep(1000); 
           //sleep(1);
        } else if (chan->running && chan->state==CHGR_STATE_RAMP) {

           pthread_mutex_lock(md->ramp_mutex);
           g_mutex_lock(chan->graph_mutex);
           chan->ndata = md->ramp_ndata; 

           if (chan->ndata>0 && md->rampvdata!=NULL)
           {
              for (i=0; i<chan->ndata; i++) {
                  chan->xdata[i] = md->rampvdata[i];                                   //zero channel must be always the ramping quantity
                  chan->ydata[i] = md->rampvdata[md->ramp_npos*(chan->channel+1) + i]; //0 was the ramping quantity, so we add 1
                  //printf("%d %g %g\n", i, chan->xdata[i], chan->ydata[i]);
              }
//              printf("we have %d ramp data points, (%g %g), (%g %g), ...  (%g %g), (%g %g)\n", chan->ndata, chan->xdata[0], chan->ydata[0], chan->xdata[1], chan->ydata[1], chan->xdata[chan->ndata-2], chan->ydata[chan->ndata-2], chan->xdata[chan->ndata-1], chan->ydata[chan->ndata-1]);
           }
           g_mutex_unlock(chan->graph_mutex);
           pthread_mutex_unlock(md->ramp_mutex);

           usleep(1000); 
           //sleep(1);
        } else 
            usleep(100000);    
            //Sleep(100);
    }
   return NULL;
}

static void
timeout_changed_cb(G_GNUC_UNUSED gpointer object,
                  GwsChangraph* chan)
{
   chan->timeout = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(chan->timeout_adj));
}

static gboolean
updateme (GwsChangraph* chan)
{
    int i;
    MiniData *md = (MiniData *)chan->md;
    if (md->closingapp) 
        return FALSE;

    g_mutex_lock(chan->graph_mutex);
    for (i=0; i<chan->ndata; i++) {
      if (fabs(chan->xdata[i])<1e-20) chan->xdata[i]=0;
      if (fabs(chan->ydata[i])<1e-20) chan->ydata[i]=0;
    }
    for (i=0; i<chan->rndata; i++) {
      if (fabs(chan->rxdata[i])<1e-20) chan->rxdata[i]=0;
      if (fabs(chan->rydata[i])<1e-20) chan->rydata[i]=0;
    }
     /*
    for (i=0; i<chan->ndata; i++) printf("%d %d %g %g\n", chan->ndata, i, chan->xdata[i], chan->ydata[i]);
    */

   
    gwy_graph_curve_model_set_data(chan->gcmodel,
                                   chan->xdata,
                                   chan->ydata,
                                   chan->ndata);
    gwy_graph_curve_model_set_data(chan->rgcmodel,
                                   chan->rxdata,
                                   chan->rydata,
                                   chan->rndata);
				   
     g_mutex_unlock(chan->graph_mutex); 
    return TRUE;
}

void gws_changraph_set_running(GwsChangraph* chan, gboolean running)
{
    chan->running = running;
}

void
gws_changraph_refresh_channels(GwsChangraph *chan)
{
    MiniData *md = (MiniData *)chan->md;
    gint i;

    if (md->closingapp) return;

    printf("refresh graph channels, setting combo to %d\n", chan->channel);

    gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(chan->mode))));

    for (i=0; i<md->nchan; i++) gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(chan->mode), (gchar *)md->descriptions[i]);
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->mode), chan->channel);

}


GwsChangraph*   
gws_changraph_new(GwyContainer *container, GwyContainer *rcontainer, gint channel, gpointer md)
{
    GtkWidget *hbox, *label, *frame, *vbox;
    gint i;
    GError *gerr = NULL;

    GwsChangraph *chan;
   
    chan = (GwsChangraph*)g_object_new(GWS_TYPE_CHANGRAPH,
                                       NULL);

    printf("new changraph with channel %d\n", channel);
    chan->running = 0;

    chan->channel = channel;
    chan->container = container;
    chan->rcontainer = rcontainer;
    chan->md = md;
    chan->vbox = gtk_vbox_new(FALSE, 0);
    chan->state = CHGR_STATE_STOPPED;
    chan->timeout = 100;
    chan->graph_mutex = g_mutex_new();


    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(chan->vbox), frame, TRUE, TRUE, 0);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    /*graph*/
    chan->gcmodel = gwy_graph_curve_model_new();
    chan->rgcmodel = gwy_graph_curve_model_new();
    chan->gmodel = gwy_graph_model_new();
    g_object_set(chan->gcmodel,
                 "mode", GWY_GRAPH_CURVE_LINE,
                 NULL);
    gwy_graph_model_add_curve(chan->gmodel, chan->gcmodel);
    gwy_graph_model_add_curve(chan->gmodel, chan->rgcmodel);
    chan->graph = GWY_GRAPH(gwy_graph_new(chan->gmodel));
    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(chan->graph), TRUE, TRUE, 0);


    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


    label = gtk_label_new("Source:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);


    chan->mode = gtk_combo_box_text_new();
    gws_changraph_refresh_channels(chan);
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->mode), chan->channel);

    g_signal_connect( G_OBJECT(chan->mode), "changed",
                      G_CALLBACK(mode_changed_cb), chan);

    gtk_box_pack_start(GTK_BOX(hbox), chan->mode, FALSE, FALSE, 0);

    label = gtk_label_new("Leveling:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    chan->level = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "None");
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "1st order line");
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->level), 0);

    g_signal_connect( G_OBJECT(chan->level), "changed",
                      G_CALLBACK(level_changed_cb), chan);

    gtk_box_pack_start(GTK_BOX(hbox), chan->level, FALSE, FALSE, 0);

    
    chan->timeout_adj = gtk_adjustment_new(chan->timeout, 1, 1000, 1, 10, 0);
    chan->timeout_spin = gtk_spin_button_new(GTK_ADJUSTMENT(chan->timeout_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(chan->timeout_spin), 0);
    g_signal_connect(chan->timeout_adj, "value-changed",
                     G_CALLBACK(timeout_changed_cb), chan);


    gtk_box_pack_start(GTK_BOX(hbox), chan->timeout_spin, FALSE, FALSE, 0);


    mode_changed_cb(GTK_COMBO_BOX(chan->mode), chan);
    level_changed_cb(GTK_COMBO_BOX(chan->level), chan);


    chan->xdata = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    chan->ydata = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    chan->rxdata = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    chan->rydata = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    chan->xdatabuf = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    chan->ydatabuf = (gdouble *)g_malloc(MAXGDATA*sizeof(gdouble));
    for (i=0; i<MAXGDATA; i++) {
	    chan->xdata[i] = chan->rxdata[i] = chan->xdatabuf[i] = 0;
	    chan->ydata[i] = chan->rydata[i] = chan->ydatabuf[i] = 0;
    }

    //chan->update_loop = g_thread_new("miniscan_changraph_thread", chanloop, chan);
    g_thread_create((GThreadFunc)chanloop, (void *)(chan), TRUE, &gerr);

/*    hThreadArray[i] = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        MyThreadFunction,       // thread function name
        pDataArray[i],          // argument to thread function 
        0,                      // use default creation flags 
        &dwThreadIdArray[i]);   // returns the thread identifier */

    g_timeout_add(10, (GSourceFunc)updateme, chan);

    return chan;

}




/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
