/*
 * changraph.h
 * Copyright (C) Petr Klapetek 2007 <pklapetek@cmi.cz>
 * 
 * changraph.h is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * changraph.h is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with callbacks.h.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#ifndef CHANGRAPH
#define CHANGRAPH

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>

G_BEGIN_DECLS

#define GWS_TYPE_CHANGRAPH            (gws_changraph_get_type())
#define GWS_CHANGRAPH(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWS_TYPE_CHANGRAPH, GwsChangraph))
#define GWS_CHANGRAPH_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), GWS_TYPE_CHANGRAPH, GwsChangraph))
#define GWS_IS_CHANGRAPH(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWS_TYPE_CHANGRAPH))
#define GWS_IS_CHANGRAPH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), GWS_TYPE_CHANGRAPH))
#define GWS_CHANGRAPH_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), GWS_TYPE_CHANGRAPH, GwsChangraphClass))

#define MAXGDATA 10000

typedef enum {
     CHGR_STATE_STOPPED = 0,
     CHGR_STATE_FREERUNNING = 1,
     CHGR_STATE_PROFILES = 2,
     CHGR_STATE_RAMP = 3
} ChanGraphState;


typedef struct _GwsChangraph      GwsChangraph;
typedef struct _GwsChangraphClass GwsChangraphClass;

struct _GwsChangraph{
    GObject *parent_instance;

    GtkWidget *vbox;
    
    ChanGraphState state;
    GThread *update_loop;

    GwyGraphCurveModel *gcmodel;
    GwyGraphCurveModel *rgcmodel;
    GwyGraphModel *gmodel;
    GwyGraph *graph;

    GtkWidget *mode;
    GtkWidget *level;
    GtkObject *timeout_adj;
    GtkWidget *timeout_spin;
 
    gint channel;
    gint leveling;
    gint timeout;
    
    gdouble *xdata;
    gdouble *ydata;
    gint ndata;

    gdouble *rxdata;
    gdouble *rydata;
    gint rndata;


    gdouble *xdatabuf;
    gdouble *ydatabuf;

    gpointer md;   
    GwyContainer *container; 
    GwyContainer *rcontainer;

    GMutex* graph_mutex;

    gboolean running;
 
    gulong completed_handler;
};

struct _GwsChangraphClass {
    GObjectClass parent_class;

    void (*selected)(GwsChangraph *changraph, gdouble *data);
};

GType           gws_changraph_get_type         (void) G_GNUC_CONST;
GwsChangraph*   gws_changraph_new              (GwyContainer *container, GwyContainer *rcontainer, gint channel, gpointer md);
void            gws_changraph_change_state     (GwsChangraph* chan, ChanGraphState state);
void            gws_changraph_set_running      (GwsChangraph* chan, gboolean running);
void            gws_changraph_refresh_channels (GwsChangraph* chan);


G_END_DECLS

#endif


/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
