/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include "channels.h"
#include "data.h"
#include "hw.h"

#define MAXCH 16

static void gws_channels_class_init          (GwsChannelsClass *klass);
static void gws_channels_init                (GwsChannels      *ttt);

GType
gws_channels_get_type (void)
{
  static GType channels_type = 0;

  if (!channels_type)
    {
      const GTypeInfo channels_info =
      {
	sizeof (GwsChannelsClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) gws_channels_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (GwsChannels),
	0,
	(GInstanceInitFunc) gws_channels_init,
      };

      channels_type = g_type_register_static (GTK_TYPE_TABLE, "GwsChannels", &channels_info, 0);
    }

  return channels_type;
}

static void
gws_channels_class_init (GwsChannelsClass *klass)
{
}

static void
gws_channels_changed_cb(GtkObject *object, GwsChannels* channels)
{
    gint i;

    channels->reverse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(channels->reverse_check));
    for (i=0; i<MAXCH; i++) channels->in[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(channels->in_check[i]));
}
void
gws_channels_set_inits(GwsChannels *channels, gboolean in[16], gboolean reverse)
{
    gint i;

    channels->reverse = reverse;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(channels->reverse_check), reverse); 

    for (i=0; i<MAXCH; i++) {
        channels->in[i] = in[i];
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(channels->in_check[i]), in[i]);
    }
    
}

static void
gws_channels_init (GwsChannels *channels)
{
  GtkWidget *label;
  gint i;
  gchar buffer[50];
  guint row = 0;

  gtk_table_resize(GTK_TABLE(channels), 4, 4);

  label = gtk_label_new("General operation:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(channels), label,
                   0, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  channels->reverse_check = gtk_check_button_new_with_mnemonic("add _reverse direction (raster mode)");  
  g_signal_connect(channels->reverse_check, "toggled",
                     G_CALLBACK(gws_channels_changed_cb), channels);
  gtk_table_attach(GTK_TABLE(channels), channels->reverse_check,
                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

  row++;

  label = gtk_label_new("Auxiliary input channels:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(channels), label,
                   0, 3, row, row+1, GTK_FILL, 0, 5, 1);
 
  row++;

  for (i=0; i<MAXCH; i++) {
     g_snprintf(buffer, sizeof(buffer), "IN _%d", i+1);
     channels->in_check[i] = gtk_check_button_new_with_mnemonic(buffer);  
     g_signal_connect(channels->in_check[i], "toggled",
                     G_CALLBACK(gws_channels_changed_cb), channels);
     gtk_table_attach(GTK_TABLE(channels), channels->in_check[i],
                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

     row++;
  }

  gtk_widget_show_all(GTK_WIDGET(channels));


}

GtkWidget*
gws_channels_new(gpointer md)
{
  GwsChannels *fv = g_object_new (gws_channels_get_type (), NULL);
  fv->md = md;

  return GTK_WIDGET(fv);
}


