/*
 * chanview.h
 * Copyright (C) Petr Klapetek 2007 <pklapetek@cmi.cz>
 * 
 * chanview.h is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * chanview.h is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with callbacks.h.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
//#include <unistd.h>
#include <stdlib.h>
//#include <utime.h>
#include "chanview.h"
#include <app/gwyapp.h>
#include <libgwymodule/gwymodule-file.h>
#include <libprocess/stats.h>
#include <libgwymodule/gwymodule.h>
#include <app/gwymoduleutils.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <libprocess/level.h>
#include <string.h>


#include <app/gwyapp.h>
#include "data.h"

#define GWS_CHANVIEW_TYPE_NAME "GwsChanview"

#define MAXPATH 5000

static void   gws_chanview_class_init        (GwsChanviewClass *klass);
static void   gws_chanview_init              (GwsChanview *chan);

enum {
    SELECTED,
    LAST_SIGNAL
};
static guint chanview_signals[LAST_SIGNAL] = { 0 };


static GObjectClass *parent_class = NULL;

GType
gws_chanview_get_type(void)
{
    static GType gws_chanview_type = 0;

    if (!gws_chanview_type) {
        static const GTypeInfo gws_chanview_info = {
            sizeof(GwsChanviewClass),
            NULL,
            NULL,
            (GClassInitFunc)gws_chanview_class_init,
            NULL,
            NULL,
            sizeof(GwsChanview),
            0,
            (GInstanceInitFunc)gws_chanview_init,
            NULL,
        };

        gws_chanview_type
            = g_type_register_static(G_TYPE_OBJECT,
                                     GWS_CHANVIEW_TYPE_NAME,
                                     &gws_chanview_info,
                                     0);
    }

    return gws_chanview_type;
}

static void
gws_chanview_class_init(GwsChanviewClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

    gwy_debug("");

    parent_class = g_type_class_peek_parent(klass);

    chanview_signals[SELECTED]
	= g_signal_new("selected",
		       G_OBJECT_CLASS_TYPE(gobject_class),
		       G_SIGNAL_ACTION | G_SIGNAL_RUN_FIRST,
		       G_STRUCT_OFFSET(GwsChanviewClass, selected),
		       NULL, NULL,
		       g_cclosure_marshal_VOID__POINTER,
		       G_TYPE_NONE, 1, G_TYPE_POINTER);


}

#if(0)
static void
selection_finished_cb(GwySelection *selection, GwsChanview *chanview)
{
    /*pass data forward*/
    gdouble sel[4];
    //printf("selection finished signal\n");
    if (!gwy_selection_get_object(selection, 0, sel)) return;
    //gwy_selection_clear(selection);
    gws_chanview_selected_signal_emit(chanview, sel);
}
#endif

static void
gws_chanview_init(GwsChanview *chan)
{
}

/*
static GwyDataField *
container_deep_transfer(GwyContainer *src, GwyContainer *dst,  const gchar *source_prefix, const gchar *dest_prefix, gboolean force)
{
    GwyDataField *dfield;
    dfield = gwy_data_field_duplicate(gwy_container_get_object_by_name(src, source_prefix));
    gwy_container_set_object_by_name(dst, dest_prefix, dfield);
//    gwy_object_unref(dfield);
//
   return dfield;
}
*/

void  gws_chanview_update(GwsChanview* chan)
{
    GwyDataField *dfield, *mydfield;   
    MiniData *md = (MiniData *)chan->md; 
    gchar key[32];
    gdouble bx, by, c;
    gint actual;
    gdouble min, max;
    gdouble *seldata;
    gint nsel;
    gint direction, channel, level;

    if (!chan->running) return;



    direction = 0;//gtk_combo_box_get_active(GTK_COMBO_BOX(chan->direction));
    channel = gtk_combo_box_get_active(GTK_COMBO_BOX(chan->mode));
    level = gtk_combo_box_get_active(GTK_COMBO_BOX(chan->level));
    
    /*copy path*/
    seldata = g_new(gdouble, 2*MAXPATH);
    nsel = gwy_selection_get_data(chan->selpath, seldata);

    pthread_mutex_lock(md->dfield_mutex);

   /*copy data again from original container   --- this leads to segfault*/
    g_snprintf(key, sizeof(key), "/%d/data", channel);
    if (!direction) {
       //gwy_container_transfer(chan->container, chan->mycontainer, key, "/0/data", TRUE);  
       //dfield = container_deep_transfer(chan->container, chan->mycontainer, key, "/0/data", TRUE);  
//       dfield = (GwyDataField *) gwy_container_get_object_by_name(chan->mycontainer, "/0/data");
         dfield = (GwyDataField *)gwy_container_get_object_by_name(chan->container, key);
    }
    else {
//       gwy_container_transfer(chan->rcontainer, chan->mycontainer, key, "/0/data", TRUE); 
//       dfield = (GwyDataField *) gwy_container_get_object_by_name(chan->rcontainer, key);
         dfield = (GwyDataField *)gwy_container_get_object_by_name(chan->container, key);
    }
    pthread_mutex_unlock(md->dfield_mutex);


    mydfield = (GwyDataField *) gwy_container_get_object_by_name(chan->mycontainer, "/0/data");   
    gwy_data_field_resample(mydfield, gwy_data_field_get_xres(dfield), gwy_data_field_get_yres(dfield), GWY_INTERPOLATION_NONE);

    pthread_mutex_lock(md->dfield_mutex);
    gwy_data_field_copy(dfield, mydfield, FALSE);
    pthread_mutex_unlock(md->dfield_mutex);

    pthread_mutex_lock(md->dfield_mutex);
    actual = CLAMP(md->actual_profile, 0, gwy_data_field_get_yres(dfield)-1);
    pthread_mutex_unlock(md->dfield_mutex);

//    gwy_container_set_object_by_name(chan->mycontainer, "/0/data", mydfield);

    gwy_set_data_preview_size(GWY_DATA_VIEW(chan->dataview), PREVIEW_SIZE_PIXELS);
    gwy_data_field_data_changed(mydfield);

    /*copy path back*/
    gwy_data_field_set_xreal(mydfield, chan->xreal);
    gwy_data_field_set_yreal(mydfield, chan->yreal);


//    printf("path back: %d %d %g %g\n", gwy_data_field_get_xres(mydfield), gwy_data_field_get_yres(mydfield), gwy_data_field_get_xreal(mydfield), gwy_data_field_get_yreal(mydfield));
    gwy_selection_set_data(chan->selpath, nsel, seldata);
    g_free(seldata);


    /*level data as requested*/
    if (level==1) { //line 1st

         gwy_data_field_fit_lines(mydfield,
                             0, 0, 0, 0,
                             1, 0, GWY_ORIENTATION_HORIZONTAL);

    } 
    else if (level==2) { //line 2nd
         gwy_data_field_fit_lines(mydfield,
                             0, 0, 0, 0,
                             2, 0, GWY_ORIENTATION_HORIZONTAL);

    } 
    else if (level==3) { //plane 1st

         gwy_data_field_fit_plane(mydfield, &c, &bx, &by);
         c = -0.5*(bx*gwy_data_field_get_xres(mydfield)
                  + by*gwy_data_field_get_yres(mydfield));

        gwy_data_field_plane_level(mydfield, c, bx, by);

    } 

    if (actual>1) {
	    min = gwy_data_field_area_get_min(mydfield, NULL, 0, 0, gwy_data_field_get_xres(mydfield), actual);
	    max = gwy_data_field_area_get_max(mydfield, NULL, 0, 0, gwy_data_field_get_xres(mydfield), actual);

	    gwy_container_set_double(chan->mycontainer, chan->key_min, min);
	    gwy_container_set_double(chan->mycontainer, chan->key_max, max);

	    gwy_layer_basic_get_range(GWY_LAYER_BASIC(chan->blayer), &min, &max);
	    gwy_color_axis_set_range(GWY_COLOR_AXIS(chan->axis), min, max);
    }

}

static gboolean
updateme (GwsChanview* chan)
{
    MiniData *md = (MiniData *)chan->md;
    if (md->closingapp) 
        return FALSE;

    gws_chanview_update(chan);
    return TRUE;
}


static void 
mode_changed_cb(GtkComboBox *combo, GwsChanview* chan)
{
    gws_chanview_update(chan);
}
static void 
direction_changed_cb(GtkComboBox *combo, GwsChanview* chan)
{
    printf("direction changed\n");
    gws_chanview_update(chan);
}

void 
gws_chanview_enable_reverse(GwsChanview *chan, gboolean enable)
{
    if (!enable) gtk_combo_box_set_active(GTK_COMBO_BOX(chan->direction), 0);
    gtk_widget_set_sensitive(GTK_WIDGET(chan->direction), enable);
}

void gws_chanview_set_running(GwsChanview* chan, gboolean running)
{  
    chan->running = running; 
}

void
gws_chanview_refresh_channels(GwsChanview *chan)
{
    MiniData *md = (MiniData *)chan->md;
    gint i;

    if (md->closingapp) return;

    printf("refresh channels!\n");

    gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(chan->mode))));

    for (i=0; i<md->nchan; i++) gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(chan->mode), (gchar *)md->descriptions[i]);
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->mode), chan->channel);

}

static GwySelection*
create_vector_layer(GwyDataView *dataview, gint id, const gchar *name,
                    gboolean editable)
{
    GwyVectorLayer *layer;
    GType type;
    gchar *s, *t;

    s = g_strconcat("GwyLayer", name, NULL);
    type = g_type_from_name(s);
    g_free(s);
    g_return_val_if_fail(type, NULL);
    g_return_val_if_fail(g_type_is_a(type, GWY_TYPE_VECTOR_LAYER), NULL);

    layer = GWY_VECTOR_LAYER(g_object_newv(type, 0, NULL));
    s = g_strdup_printf("/%d/select/%s", id, name);
    t = strrchr(s, '/');
    g_assert(t);
    t++;
    *t = g_ascii_tolower(*t);
    gwy_vector_layer_set_selection_key(layer, s);
    g_free(s);
    gwy_vector_layer_set_editable(layer, editable);
    gwy_data_view_set_top_layer(dataview, layer);

    return gwy_vector_layer_ensure_selection(layer);
}


GwsChanview*   
gws_chanview_new(GwyContainer *container, GwyContainer *rcontainer, gint channel, gpointer md)
{
    GtkWidget *hbox, *label, *frame, *vbox;
    GwsChanview *chan;
    GwyPixmapLayer *layer;
    gchar key[32];
    //GwyVectorLayer *vlayer;
    //const gchar *prefix;
   
    chan = (GwsChanview*)g_object_new(GWS_TYPE_CHANVIEW,
                                                NULL);

    chan->vbox = gtk_vbox_new(FALSE, 0);
    chan->running = 0;
    chan->field_mutex = g_mutex_new();

    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(chan->vbox), frame, TRUE, TRUE, 0);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);


    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    

    chan->channel = channel;
    chan->md = md;
    chan->container = container;
    chan->rcontainer = rcontainer;
    chan->mycontainer = gwy_container_new();
    chan->dfield = gwy_data_field_new(10, 10, 10, 10, 1);
    chan->xreal = 10;
    chan->yreal = 10;

    g_snprintf(key, sizeof(key), "/0/data");
    gwy_container_set_object_by_name(chan->mycontainer, key, chan->dfield);

    chan->dataview = GWY_DATA_VIEW(gwy_data_view_new(chan->mycontainer));

    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(chan->dataview), TRUE, TRUE, 0);

    gwy_data_view_set_data_prefix(GWY_DATA_VIEW(chan->dataview), key);

    chan->blayer = layer = gwy_layer_basic_new();
    g_object_set(layer,
                 "data-key", "/0/data",
                 "gradient-key", "/0/base/palette",
                 NULL);

    gwy_data_view_set_base_layer(GWY_DATA_VIEW(chan->dataview), layer);
    gwy_pixmap_layer_set_data_key(layer, "/0/data");    

    gwy_set_data_preview_size(GWY_DATA_VIEW(chan->dataview), PREVIEW_SIZE_PIXELS);

//    chan->selpath = create_vector_layer(GWY_DATA_VIEW(chan->dataview),
//                                             0, "Path", TRUE);
    chan->selpath = create_vector_layer(GWY_DATA_VIEW(chan->dataview),
                                             0, "Point", FALSE);
     gwy_selection_set_max_objects(chan->selpath, 1);

//    g_object_set(chan->selpath,
//                 "editable", 0,
//                  NULL);
    gws_chanview_hidepath(chan);

    chan->axis = gwy_color_axis_new(GTK_ORIENTATION_VERTICAL);
    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(chan->axis), FALSE, FALSE, 0);


    g_snprintf(key, sizeof(key), "/0/base/min");
    chan->key_min = g_quark_from_string(key);
    g_snprintf(key, sizeof(key), "/0/base/max");
    chan->key_max = g_quark_from_string(key);



/*
    vlayer = g_object_new(g_type_from_name("GwyLayerRectangle"), NULL);
    gwy_vector_layer_set_selection_key(vlayer, "/0/select/zoom/rect");
    gwy_data_view_set_top_layer(GWY_DATA_VIEW(chan->dataview), vlayer);
    chan->selection = gwy_vector_layer_ensure_selection(vlayer);
    g_signal_connect(chan->selection, "finished",
		               G_CALLBACK(selection_finished_cb), chan);
  */  


    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


    label = gtk_label_new("Source:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);


    chan->mode = gtk_combo_box_text_new();
    gws_chanview_refresh_channels(chan);
    g_signal_connect( G_OBJECT(chan->mode), "changed",
                      G_CALLBACK(mode_changed_cb), chan);

    gtk_box_pack_start(GTK_BOX(hbox), chan->mode, FALSE, FALSE, 0);

    label = gtk_label_new("Leveling:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    chan->level = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "None");
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "1st order line");
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "2nd order line");
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->level), "Plane");
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->level), 0);

    g_signal_connect( G_OBJECT(chan->level), "changed",
                      G_CALLBACK(mode_changed_cb), chan);

    gtk_box_pack_start(GTK_BOX(hbox), chan->level, FALSE, FALSE, 0);

    chan->direction = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->direction), "Fw");
    gtk_combo_box_append_text(GTK_COMBO_BOX(chan->direction), "Rev");
    gtk_combo_box_set_active(GTK_COMBO_BOX(chan->direction), 0);

    g_signal_connect( G_OBJECT(chan->direction), "changed",
                      G_CALLBACK(direction_changed_cb), chan);

    gtk_box_pack_start(GTK_BOX(hbox), chan->direction, FALSE, FALSE, 0);


    mode_changed_cb(GTK_COMBO_BOX(chan->mode), chan);

    g_timeout_add(1000, (GSourceFunc)updateme, chan);


    return chan;

}

void  
gws_chanview_showpath(GwsChanview* chan, gdouble *xydata, gint ndata)
{
    gint n = MIN(ndata, MAXPATH);
    gint i;
//     gint xres, yres;
//     gdouble xreal, yreal;

//    GwyDataField *mydfield = gwy_container_get_object_by_name(chan->mycontainer, "/0/data");

//     xres = gwy_data_field_get_xres(mydfield);
//     yres = gwy_data_field_get_yres(mydfield);
//     xreal = gwy_data_field_get_xreal(mydfield);
//     yreal = gwy_data_field_get_yreal(mydfield);

    gdouble *xy = g_new(gdouble, 2*n);

//    printf("show path: %d of %d points  %d x %d   %g  %g\n", n, ndata, xres, yres, xreal, yreal);

    for (i = 0; i < n; i++) {
//	    xy[2*i] = i;
//	    xy[2*i+1] = 0.01*i*i;
	    xy[2*i] = xydata[2*i];//*xres/xreal;
            xy[2*i+1] = xydata[2*i+1];//*yres/yreal;
	    //if (i<5) printf("%g %g\n", xy[2*i], xy[2*i+1]);
    }

    gwy_selection_set_data(chan->selpath, n, xy);

    g_free(xy);

}

void  
gws_chanview_hidepath(GwsChanview* chan)
{
    gint i, n = 2;
    gdouble *xy = g_new(gdouble, 2*n);
 
    for (i = 0; i < 2*n; i++) xy[i] = 0;

//    printf("hide path\n");
    gwy_selection_clear(chan->selpath);
    //gwy_selection_set_data(chan->selpath, n, xy);

    g_free(xy);
    
}

void 
gws_chanview_set_ranges(GwsChanview* chan, gdouble xreal, gdouble yreal)
{
    GwyDataField *mydfield = gwy_container_get_object_by_name(chan->mycontainer, "/0/data");
//    printf("set ranges to %g %g\n", xreal, yreal);
    gwy_data_field_set_xreal(mydfield, xreal);
    gwy_data_field_set_yreal(mydfield, yreal);
   
    chan->xreal = xreal;
    chan->yreal = yreal; 
}


void
data_line_resample_or_average(GwyDataLine *data_line,
                              gint res,
                              GwyInterpolationType interpolation)
{
  /*  gint i, j, from, to;
    gdouble ratio, *bdata;

    g_return_if_fail(GWY_IS_DATA_LINE(data_line));
    if (res == data_line->res)
        return;

    ratio = ((gdouble)data_line->res/(gdouble)res);
    if (ratio<2) {
	gwy_data_line_resample(data_line, res, interpolation);
	return;
    }
   
    bdata = g_new(gdouble, res);
    
    for (i=0; i<res; i++)
    {
	bdata[i] = 0;
	from = i*ratio;
	to = (i+1)*ratio;
	if (to >= data_line->res) to = data_line->res - 1;
	if (from >= data_line->res) from = data_line->res - 1;
	for (j=from; j<to; j++) {
	    bdata[i]+=data_line->data[j];
	}
	bdata[i]/=(to-from);
    }
    
    g_free(data_line->data);
    data_line->data = bdata;
    data_line->res = res;
*/
}



/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
