/*
 * chanview.h
 * Copyright (C) Petr Klapetek 2007 <pklapetek@cmi.cz>
 * 
 * chanview.h is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * chanview.h is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with callbacks.h.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */


#ifndef CHANVIEWH
#define CHANVIEWH

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>


G_BEGIN_DECLS

#define GWS_TYPE_CHANVIEW            (gws_chanview_get_type())
#define GWS_CHANVIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GWS_TYPE_CHANVIEW, GwsChanview))
#define GWS_CHANVIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), GWS_TYPE_CHANVIEW, GwsChanview))
#define GWS_IS_CHANVIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GWS_TYPE_CHANVIEW))
#define GWS_IS_CHANVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), GWS_TYPE_CHANVIEW))
#define GWS_CHANVIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), GWS_TYPE_CHANVIEW, GwsChanviewClass))


typedef struct _GwsChanview      GwsChanview;
typedef struct _GwsChanviewClass GwsChanviewClass;

struct _GwsChanview{
    GObject *parent_instance;

    GtkWidget *vbox;

    GwyDataView *dataview;
    GtkWidget *axis;

    GtkWidget *mode;
    GtkWidget *level;
    GtkWidget *direction;
 
    GwyContainer *container;
    GwyContainer *rcontainer;
    gpointer md;
 
    GwyContainer *mycontainer;
    GwyDataField *dfield;
    GwySelection *selection;
    gint channel;

    GwyPixmapLayer *blayer;
    GQuark key_min;
    GQuark key_max;

    GMutex* field_mutex;

    gint *index;
    gdouble *data;
    gdouble *dindex;
    gint xres;
    gint yres;
    gdouble xreal;
    gdouble yreal;

    GwySelection *selpath;

    gboolean running;

    gulong completed_handler;
};

struct _GwsChanviewClass {
    GObjectClass parent_class;

    void (*selected)(GwsChanview *chanview, gdouble *data);
};

GType      gws_chanview_get_type       (void) G_GNUC_CONST;
GwsChanview*   gws_chanview_new          (GwyContainer *container, GwyContainer *rcontainer, gint channel, gpointer md);

void  gws_chanview_showpath(GwsChanview* chan, gdouble *xydata, gint ndata);
void  gws_chanview_hidepath(GwsChanview* chan);
void  gws_chanview_enable_reverse(GwsChanview *chan, gboolean enable);
void  gws_chanview_set_ranges(GwsChanview* chan, gdouble xreal, gdouble yreal);
void  gws_chanview_set_running(GwsChanview* chan, gboolean running);
void  gws_chanview_update(GwsChanview* chan);
void  gws_chanview_refresh_channels(GwsChanview* chan);

G_END_DECLS
#endif


/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
