
#include <stdlib.h>
#include <string.h>
#include "data.h"
#include "hw.h"



void log_monitor_cb(GFileMonitor *monitor, GFile *file,
                        GFile *other_file,
                        GFileMonitorEvent event_type,
                        MiniData *md)
{
    GError *error;
    gboolean read_file_status; 
    gchar *buffer;
    GtkTextIter end;

    if (event_type==G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {

       read_file_status = g_file_get_contents(md->logfile, &buffer, NULL, &error);

       if (read_file_status == FALSE) {
	       g_error("error opening file: %s\n", error && error->message ? error->message : "No Detail");
	       return;
       }
       gtk_text_buffer_set_text (md->tblog, buffer,-1);
       gtk_text_buffer_get_end_iter(md->tblog, &end);
       gtk_text_buffer_place_cursor(md->tblog, &end);
       //gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(md->viewlog), &end, 0, 0, 0, 0);

       g_free(buffer);
    }

}

void set_preview_channel_descriptions(MiniData *md) //FIXME add lifts
{
    gint k, n;
    gchar buffer[50];
    gint i;

    if (md->descriptions) {
    for (i=0; i<md->nchan; i++) g_free(md->descriptions[i]);
       g_free(md->descriptions);
    }

    i = 0;
    if (md->c_pr_x) i++;
    if (md->c_pr_y) i++;
    if (md->c_pr_z) i++;
    if (md->c_pr_e) i++;
    if (md->c_pr_ts) i++;
    if (md->c_sc_a1) i++;
    if (md->c_sc_p1) i++;
    if (md->c_sc_a2) i++;
    if (md->c_sc_p2) i++;
    for (n=0; n<16; n++) if (md->c_sc_in[n]) i++;
    for (k=0; k<md->nlift; k++)
    {
       if (md->c_sc_a1) i++;
       if (md->c_sc_p1) i++;
       if (md->c_sc_a2) i++;
       if (md->c_sc_p2) i++;
       for (n=0; n<16; n++) if (md->c_sc_in[n]) i++; 
    }

    md->nchan = i;


    md->descriptions = (gchar **)malloc(md->nchan*sizeof(gchar *));
    i = 0;
    if (md->c_pr_x) md->descriptions[i++] = g_strdup("X pos");
    if (md->c_pr_y) md->descriptions[i++] = g_strdup("Y pos");
    if (md->c_pr_z) md->descriptions[i++] = g_strdup("Z piezo");
    if (md->c_pr_e) md->descriptions[i++] = g_strdup("Error");
    if (md->c_pr_ts) md->descriptions[i++] = g_strdup("Timestamp");
    if (md->c_sc_a1) md->descriptions[i++] = g_strdup("Amplitude 1");
    if (md->c_sc_p1) md->descriptions[i++] = g_strdup("Phase 1");
    if (md->c_sc_a2) md->descriptions[i++] = g_strdup("Amplitude 2");
    if (md->c_sc_p2) md->descriptions[i++] = g_strdup("Phase 2");
    for (n=0; n<16; n++) 
       if (md->c_sc_in[n]) {
          g_snprintf(buffer, 50, "In %d", n+1);
          md->descriptions[i++] = g_strdup(buffer);
       }
    for (k=0; k<md->nlift; k++)
    {
       if (md->c_sc_a1) md->descriptions[i++] = g_strdup_printf("A1, lift %d", k);
       if (md->c_sc_p1) md->descriptions[i++] = g_strdup_printf("Ph1, lift %d", k);
       if (md->c_sc_a2) md->descriptions[i++] = g_strdup_printf("A2, lift %d", k);
       if (md->c_sc_p2) md->descriptions[i++] = g_strdup_printf("Ph2, lift %d", k);
       for (n=0; n<16; n++) if (md->c_sc_in[n]) md->descriptions[i++] = g_strdup_printf("In %d, lift %d", n+1, k);
    }

    for (i=0; i<md->nchan; i++) printf("%d %d %s\n", md->nchan, i, md->descriptions[i]);
}

void update_preview_channels_data(MiniData *md) //FIXME add lifts
{
    gint i;
    gchar key[32];

    set_preview_channel_descriptions(md);

    for (i=0; i<50; i++) {
       if (md->dfield[i]) gwy_object_unref(md->dfield[i]);
    }

    for (i=0; i<md->nchan; i++) {
       md->dfield[i] = gwy_data_field_new(PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, 1);
       g_snprintf(key, sizeof(key), "/%d/data", i);
       gwy_container_set_object_by_name(md->container, key, md->dfield[i]);
       g_snprintf(key, sizeof(key), "/%d/data/title", i);
       gwy_container_set_string_by_name(md->container, key, (guchar *)g_strdup(md->descriptions[i]));

    }
    if (md->c_sc_reverse) {
       for (i=0; i<50; i++) {
          if (md->rdfield[i]) gwy_object_unref(md->rdfield[i]);
       }

       for (i=0; i<md->nchan; i++) {
          md->rdfield[i] = gwy_data_field_new(PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, 1); 
          g_snprintf(key, sizeof(key), "/%d/data", i);
          gwy_container_set_object_by_name(md->rcontainer, key, md->rdfield[i]);
          g_snprintf(key, sizeof(key), "/%d/data/title", i);
          gwy_container_set_string_by_name(md->rcontainer, key, (guchar *)g_strdup(md->descriptions[i]));
       }
 
    }


}


gboolean init_data(MiniData *md, gchar* ini_folder, gint preview_size, void(*start_read_threads_func)(void*))
{
    gint i;
    printf("init data\n");

    md->running = FALSE;

    md->state = GWS_NOTHING;
    md->filename = NULL;
    md->project_filename = NULL;

    md->sock = -1;

    md->container = gwy_container_new();
    md->rcontainer = gwy_container_new();
    md->project = gwy_container_new();

    md->threshold = 0.0;
    md->filter1 = 12;
    md->filter2 = 12;
    
    md->xres = md->yres = md->rxres = md->ryres = 500;
    md->scanning = FALSE;
    md->approaching = FALSE;
    md->creating_path = FALSE;
    md->npos = 0;  //number of xy data allocated
    md->scan_start_time = 0;
    md->scan_end_time = 0;
    md->preview_step = 10000;

    md->xrange = md->yrange = 100e-6;
    md->xreal = md->yreal = md->rxreal = md->ryreal = 10e-6;
    md->xoff = md->yoff = md->rxoff = md->ryoff = 0;
    md->profile = md->rprofile = 0;
    md->rotate = md->rrotate = 0;
    md->actual_profile = 0;
    md->actual_profile_candidate = 0;
    md->nvals = 1000;   //number of values for e.g. spiral scan
    md->order = 5;      //order of refinement e.g. for space filling scan
    md->noctaves = 3;   //octave scan parameter
    md->nprofiles = 100; //profiles for 1d octave scan

    md->ndata = 0;  //number of zdata really measured
    md->xypos = NULL;
    md->vdata = NULL;

    md->rampvdata = NULL;
    md->ramp_npos = 0;
    md->ramp_ndata = 0;

    md->rndata = 0;  //number of zdata really measured
    md->rxypos = NULL;
    md->rvdata = NULL;

    for (i=0; i<16; i++) md->dac[i] = 0;

    md->wait = 0;
    md->speed = 100;
    md->pathval = PATH_REGULAR;

    md->pidp = 0.5;
    md->pidi = 0.3;
    md->pidd = 0;
    md->fmpidp = 0.1;
    md->fmpidi = 0.1;
    md->fmpidd = 0;
    md->fmsetpoint = 42000;
    md->zpiezo = 0;
    md->f = 0;
    md->exc1 = 0.9;
    md->exc2 = 0.9;
    md->freq_from = 30;
    md->freq_to = 50;
    md->freq_step = 0.1;
    md->freq_delay = 10;
    md->freq_aout0 = 0;

    md->mode = 0;
    md->rampmode = md->ramp_quantity = 0;
    md->ramp_do = 0;

    md->lift1_value = 0;
    md->lift2_value = 0;
    md->lift3_value = 0;
    md->lift4_value = 0;
    md->lift5_value = 0;

    md->lift1_height = 0;
    md->lift2_height = 0;
    md->lift3_height = 0;
    md->lift4_height = 0;
    md->lift5_height = 0;

    md->kpfm_frequency = 0;
    md->kpfm_amplitude = 0;
    md->kpfm_offset = 0;
    md->kpfm_routing = 0;
    md->kpfm_phase = 0;
    md->kpfm_bw = 0;

    md->c_sc_reverse = FALSE;

    md->litho_routing = 0;
   
    md->descriptions = NULL;

    md->c_pr_x = TRUE;
    md->c_pr_y = TRUE;
    md->c_pr_z = TRUE;
    md->c_pr_e = TRUE;
    md->c_pr_ts = TRUE;
    md->c_sc_a1 = FALSE;
    md->c_sc_a2 = FALSE;
    md->c_sc_p1 = FALSE;
    md->c_sc_p2 = FALSE;
    for (i=0; i<16; i++) md->c_sc_in[i] = FALSE;

    md->c_sc_in[0] = TRUE;
    md->c_sc_in[1] = TRUE;

    md->closingapp = FALSE;

    md->xpos_actual = md->ypos_actual = 0;
    md->error_actual = md->zpiezo_actual = md->in1_actual = md->in2_actual = 0;

    md->hw_start_time = 0.0f;

    md->preview_df = (GwyDataField **)g_malloc(50 * sizeof(GwyDataField *));
    md->preview_data = (gdouble **)g_malloc(50 * sizeof(gdouble *));

    md->cda = NULL;
    md->brick_z1 = NULL;
    md->brick_e1 = NULL;
    md->brick_i1 = NULL;
    md->brick_z2 = NULL;
    md->brick_e2 = NULL;
    md->brick_i2 = NULL;

    md->ramp_zres = 100;
    md->ramp_zfrom = 0;
    md->ramp_zto = 5;
    md->ramp_time = 800;
    md->ramp_starttime = 100;
    md->ramp_peaktime = 100;
    md->ramp_xpos = md->xreal/2;
    md->ramp_ypos = md->yreal/2;

    md->lua_size = 10000;

    md->in_setup = 1;
    
    md->out1rule = 0;
    md->out2rule = 0;
    md->out1value = 0;
    md->out2value = 0;
    md->tipv = 0;
    md->samplev = 0;

    md->nlift = 0;

    md->lost = 1;

//    md->cap_mutex = g_mutex_new();
//    md->zpiezo_mutex = g_mutex_new();
//    md->scan_mutex = g_mutex_new();
//    md->ramp_mutex = g_mutex_new();
//    md->dfield_mutex = g_mutex_new();
//    md->rdfield_mutex = g_mutex_new();
//    md->approach_mutex = g_mutex_new();
//    md->create_path_mutex = g_mutex_new();
//    md->sock_mutex = g_mutex_new();
//    md->hw_start_time_mutex = g_mutex_new();

    md->cap_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->zpiezo_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->scan_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->ramp_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->dfield_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->rdfield_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->approach_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->create_path_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->sock_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    md->hw_start_time_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));

    pthread_mutex_init(md->cap_mutex, NULL);
    pthread_mutex_init(md->zpiezo_mutex, NULL);
    pthread_mutex_init(md->scan_mutex, NULL);
    pthread_mutex_init(md->ramp_mutex, NULL);
    pthread_mutex_init(md->dfield_mutex, NULL);
    pthread_mutex_init(md->rdfield_mutex, NULL);
    pthread_mutex_init(md->approach_mutex, NULL);
    pthread_mutex_init(md->create_path_mutex, NULL);
    pthread_mutex_init(md->sock_mutex, NULL);
    pthread_mutex_init(md->hw_start_time_mutex, NULL);

    md->logfile = g_build_filename(g_get_home_dir(), ini_folder, "log.log", NULL);

    md->dfield = (GwyDataField **)g_malloc(50*sizeof(GwyDataField *));
    for (i=0; i<50; i++) md->dfield[i] = NULL;

    md->rdfield = (GwyDataField **)g_malloc(50*sizeof(GwyDataField *));
    for (i=0; i<50; i++) md->rdfield[i] = NULL;

    md->phaseshift1 = md->phaseshift2 = 0;
   

    /*initialise preview data*/
    update_preview_channels_data(md);



    return TRUE;
}

void start_logging_and_threads(MiniData *md)
{
    GError *error = NULL;

    /*initialise monitoring the log file*/
    md->log_monitor = g_file_monitor_file (g_file_new_for_path(md->logfile),
                                           G_FILE_MONITOR_NONE,
                                           NULL,
                                           &error);

    if (error) 
        printf("Error while connecting monitor to log file: %s\n", error->message);
    else
        g_signal_connect(md->log_monitor, "changed", G_CALLBACK(log_monitor_cb), md);

    writelog(md, "data initialized.");

    md->hw_loop = NULL;
    //start_read_threads_func(md);
    start_read_threads(md);
    writelog(md, "read threads started.");

}

void free_data(MiniData *md)
{
    if (NULL != md->logfile) {
        g_free(md->logfile);
        md->logfile = NULL;
    }

    if (NULL != md->xypos) {
        g_free(md->xypos);
        md->xypos = NULL;
    }

    if (NULL != md->vdata) {
        g_free(md->vdata);
        md->vdata = NULL;
    }

    if (NULL != md->rxypos) {
        g_free(md->rxypos);
        md->rxypos = NULL;
    }

    if (NULL != md->rvdata) {
        g_free(md->rvdata);
        md->rvdata = NULL;
    }


    if (NULL != md->dfield) {
        g_free(md->dfield);
        md->dfield = NULL;
    }

    if (NULL != md->rdfield) {
        g_free(md->rdfield);
        md->rdfield = NULL;
    }


    if (NULL != md->view) {
        g_free(md->view);
        md->view = NULL;
    }

    if (NULL != md->graph) {
        g_free(md->graph);
        md->graph = NULL;
    }

    if (NULL != md->preview_df) {
        g_free(md->preview_df);
        md->preview_df = NULL;
    }

    if (NULL != md->preview_data) {
        g_free(md->preview_data);
        md->preview_data = NULL;
    }
}

void writelog(MiniData *md, gchar *message)
{
    FILE *fw;
    gchar *format;
    GDateTime *datetime;    

    fw = fopen(md->logfile, "a");
    if (!fw) {
        fprintf(stderr, "Error: cannot open log file %s\n", md->logfile);
        return;
    }
    //fopen_s(&fw, md->logfile, "a");

    datetime = g_date_time_new_now_local();
    format = g_date_time_format(datetime, "%T");
    fprintf(fw, "%s: %s\n", format, message);
    if (NULL != format)
        g_free(format);

    fclose(fw);

    g_date_time_unref(datetime); 
}

void writelog_date(MiniData *md)
{
    FILE *fw;
    gchar *format;
    gchar *format_date;
    GDateTime *datetime;

    fw = fopen(md->logfile, "a");
    if (!fw) {
        fprintf(stderr, "Error: cannot open log file %s\n", md->logfile);
        return;
    }
     //fopen_s(&fw, md->logfile, "a");

    datetime = g_date_time_new_now_local();
    format = g_date_time_format(datetime, "%T");
    format_date = g_date_time_format(datetime, "%F");
    fprintf(fw, "%s: %s\n", format, format_date);
    if (NULL != format)
        g_free(format);
    if (NULL != format_date)
        g_free(format_date);

    fclose(fw);

    g_date_time_unref(datetime);
}


/*read only actual value and store it, functions below can thread safe return it back. All the rest is done within GUI or other threads that will be created*/

void get_actual_pos(MiniData *md, gdouble *xpos, gdouble *ypos)
{
    pthread_mutex_lock(md->cap_mutex);
    *xpos = md->xpos_actual;
    *ypos = md->ypos_actual;
    pthread_mutex_unlock(md->cap_mutex);
}

void get_actual_in(MiniData *md, gdouble *in)
{
    gint i;
    pthread_mutex_lock(md->zpiezo_mutex);
    for (i=0; i<16; i++) in[i] = md->in_actual[i];
    pthread_mutex_unlock(md->zpiezo_mutex);
}

void get_actual_zdata(MiniData *md, gdouble *zpiezo, gdouble *error, gdouble *in1, gdouble *in2, gdouble *time, gdouble *a1, gdouble *p1)
{
    pthread_mutex_lock(md->zpiezo_mutex);
    *zpiezo = md->zpiezo_actual;
    *error = md->error_actual;
    *in1 = md->in1_actual;
    *in2 = md->in2_actual;
    *time = md->time_actual;
    *a1 = md->a1_actual;
    *p1 = md->p1_actual;
    pthread_mutex_unlock(md->zpiezo_mutex);
}


void get_actual_hw_start_time(MiniData *md, gdouble *time)
{
    pthread_mutex_lock(md->hw_start_time_mutex);
    *time = md->hw_start_time;
    pthread_mutex_unlock(md->hw_start_time_mutex);
}

void get_actual_scan_times(MiniData *md, gdouble *start_time, gdouble *end_time)
{
    pthread_mutex_lock(md->scan_mutex);
    *start_time = md->scan_start_time;
    *end_time = md->scan_end_time;
    pthread_mutex_unlock(md->scan_mutex);
}

void get_actual_preview_step(MiniData *md, gint *preview_step)
{
    pthread_mutex_lock(md->scan_mutex);
    *preview_step = md->preview_step;
    pthread_mutex_unlock(md->scan_mutex);
}


/*universal readout function. Arranges live results same way as the channels are arranged
and picks the right one*/
void get_actual_whatever(MiniData *md, gdouble *val, gint channel)
{
    gint i, n;
    gdouble x, y, z, e, ts, a1, p1, a2, p2, in[16], adc1, adc2;
    gdouble data[50];

    for (i=0; i<50; i++) data[i] = 0;
    for (i=0; i<16; i++) in[i] = 0;
    a2 = p2 = 0;

    get_actual_zdata(md, &z, &e, &adc1, &adc2, &ts, &a1, &p1);
    get_actual_pos(md, &x, &y);
    get_actual_in(md, in);

    n = 0;
    if (md->c_pr_x) data[n++] = x;
    if (md->c_pr_y) data[n++] = y;
    if (md->c_pr_z) data[n++] = z;
    if (md->c_pr_e) data[n++] = e;
    if (md->c_pr_ts) data[n++] = ts;
    if (md->c_sc_a1) data[n++] = a1;
    if (md->c_sc_p1) data[n++] = p1;
    if (md->c_sc_a2) data[n++] = a2;
    if (md->c_sc_p2) data[n++] = p2;
    for (i=0; i<16; i++) if (md->c_sc_in[i]) data[n++] = in[i];


    *val = data[channel];
}
