
#ifndef MINISCAN_DATA
#define MINISCAN_DATA

#include <gtk/gtk.h>
#include <libgwydgets/gwydgets.h>
#include <libprocess/gwyprocess.h>
#include <libgwyddion/gwycontainer.h>
#include "chanview.h"
#include "changraph.h"

#define PREVIEW_SIZE_PIXELS 200

typedef enum {
   PATH_REGULAR = 0,
   PATH_SPACE_FILLING = 1,
   PATH_SPIRAL = 2,
   PATH_RANDOM = 3,
   PATH_OCTAVE_1D = 4,
   PATH_OCTAVE_2D = 5,
   PATH_WICHMANN = 6,
   PATH_REFINE = 7,
   PATH_ALTERNATEBLOCK = 8,
   MAXPATHS = 9
} ScanPath;

typedef enum {
   GWS_NOTHING = 0,
   GWS_APPROACHING = 1,
   GWS_SCANNING = 2
} ScanState;


typedef enum {
    GWS_MODE_OFF           = 0,
    GWS_MODE_PROPORTIONAL  = 1,
    GWS_MODE_NCAMPLITUDE   = 2,
    GWS_MODE_NCPHASE       = 3,
    GWS_MODE_AKIYAMA       = 4,
    GWS_MODE_NENOPROBE     = 5,
    GWS_MODE_STM           = 6
} GwsMode;

typedef enum {
    GWS_RAMP_ZPIEZO        = 0,
    GWS_RAMP_TIPV          = 1,
    GWS_RAMP_SAMPLEV       = 2,
    GWS_RAMP_OUT1          = 3,
    GWS_RAMP_OUT2          = 4
} GwsRamp;

typedef enum {
    GWS_LIFT_FEEDBACK      = 0,
    GWS_LIFT_PROFILE       = 1,
    GWS_LIFT_LPROFILE      = 2,
    GWS_LIFT_IMAGE         = 3,
    GWS_LIFT_LIMAGE        = 4,
    GWS_LIFT_POINT         = 5
} GwsLift;

typedef enum {
    GWS_LIFTSWITCH_NOTHING    = 0,
    GWS_LIFTSWITCH_TIPV       = 1,
    GWS_LIFTSWITCH_SAMPLEV    = 2,
    GWS_LIFTSWITCH_OUT1       = 3,
    GWS_LIFTSWITCH_OUT2       = 4
} GwsLiftSwitch;

typedef enum {
    GWS_RULE_NOTHING  = 0,
    GWS_RULE_X        = 1,
    GWS_RULE_Y        = 2,
    GWS_RULE_Z        = 3,
    GWS_RULE_IN1      = 4,
    GWS_RULE_IN2      = 5
} GwsOutRule;

typedef enum {
    GWS_ROUTE_NOWHERE  = 0,
    GWS_ROUTE_TIP      = 1,
    GWS_ROUTE_SAMPLE   = 2,
    GWS_ROUTE_OUT1     = 3,
    GWS_ROUTE_OUT2     = 4
} GwsRoute;

typedef enum {
    GWS_FMLIMIT_0  = 0,
    GWS_FMLIMIT_1  = 1,
    GWS_FMLIMIT_2  = 2,
    GWS_FMLIMIT_3  = 3,
    GWS_FMLIMIT_4  = 4,
    GWS_FMLIMIT_5  = 5,
    GWS_FMLIMIT_6  = 6,
    GWS_FMLIMIT_7  = 7
} GwsFMLimitFactor;

typedef enum {
    GWS_PHASELIMIT_0  = 0,
    GWS_PHASELIMIT_1  = 1,
    GWS_PHASELIMIT_2  = 2,
    GWS_PHASELIMIT_3  = 3,
    GWS_PHASELIMIT_4  = 4,
    GWS_PHASELIMIT_5  = 5,
    GWS_PHASELIMIT_6  = 6,
    GWS_PHASELIMIT_7  = 7
} GwsPhaseLimitFactor;


typedef struct {
  
    /*data*/
    ScanState state;

    gchar *filename;
    gchar *project_filename;
    GwyContainer *container;
    GwyContainer *rcontainer;
    GwyContainer *project;
    GwyDataField **dfield;
    GwyDataField **rdfield;
    gchar *logfile;

    gint sock;
    gint sock_control_message;
    gchar *server;
    gint port;

    gint mode;
    gint rampmode;

    gint in_setup;
    gint page;

    gint last_project_index;

    gint lost;

    GFileMonitor *log_monitor;

    gdouble xrange;
    gdouble yrange;
    gdouble zrange;
    gint hrdac_range;
    gint zpiezo_hrdac_range;
    gint xymode;
    gboolean zpiezo_hrdac;
    gboolean swap_proportional;
    gboolean swap_nenoprobe;
    gboolean swap_zpiezo;

    gint filter1;
    gint filter2;
    gint phaseshift1;
    gint phaseshift2;

    /*general non-equidistant data*/
    gdouble *xypos; 
    gdouble *vdata;
    gint ndata; //real number of measured data

    gdouble *rxypos;  //reverse xy positions
    gdouble *rvdata;  //reverse data buffer
    gint rndata;      //reverse real number of measured data

    /*preview and scan acquire settings*/
    gboolean c_pr_x;
    gboolean c_pr_y;
    gboolean c_pr_z;
    gboolean c_pr_e;
    gboolean c_pr_ts;
    gboolean c_sc_a1;
    gboolean c_sc_p1;
    gboolean c_sc_a2;
    gboolean c_sc_p2;
    gboolean c_sc_in[16];
    gboolean c_sc_reverse;

    /*z pos data*/
    gboolean profileonly;
    gboolean rprofileonly;

    /*adaptive regular scan data*/
    gint xres;
    gint yres;
    gdouble xreal;
    gdouble yreal;
    gdouble xoff;
    gdouble yoff;
    gdouble rotate;
    gint profile;

    /*raster regular scan data*/
    gint rxres;
    gint ryres;
    gdouble rxreal;
    gdouble ryreal;
    gdouble rxoff;
    gdouble ryoff;
    gdouble rrotate;
    gint rprofile;
  
    /*adaptive scan data*/
    gint nvals;
    gint order;
    gint nprofiles;
    gint noctaves;

    /*global scan data*/
    gint wait;
    gdouble speed;

    /*pause scan state*/
    gint pause;

    /*feedback data*/
    gdouble pidp;
    gdouble pidi;
    gdouble pidd;

    gdouble fmpidp;
    gdouble fmpidi;
    gdouble fmpidd;
    gdouble fmsetpoint;
    gint fm_frequency_limit;
    gint fm_phase_limit;
    gboolean phasefilter;
    gboolean amfm;

    gdouble f;
    gdouble exc1;
    gdouble exc2;
    gdouble zpiezo;

    gdouble freq_from;
    gdouble freq_to;
    gdouble freq_step;
    gint freq_delay;
    gdouble freq_aout0;

    /*io data*/
    gdouble tipv;
    gdouble samplev;
    gint out1rule;
    gdouble out1value;
    gint out2rule;
    gdouble out2value;

    /*slow dac data*/
    gdouble dac[16];

    /*force volume data*/
    gint ramp_quantity; //now equal to rampmode
    gint ramp_zres;
    gint ramp_npos; //allocated number
    gdouble ramp_zfrom;
    gdouble ramp_zto;
    gdouble ramp_time;
    gdouble ramp_starttime;
    gdouble ramp_peaktime;
    gdouble ramp_xpos;
    gdouble ramp_ypos;
    gboolean ramp_do;
    gint ramp_ndata; //measured number
    gdouble *rampvdata;

    /*kpfm data*/
    gdouble kpfm_frequency;
    gdouble kpfm_amplitude;
    gdouble kpfm_offset;
    gint kpfm_routing;
    gdouble kpfm_phase;
    gdouble kpfm_bw;

    /*lift mode data*/
    gint liftmode;
    gdouble nlift;
    gdouble lift1_height;
    gdouble lift2_height;
    gdouble lift3_height;
    gdouble lift4_height;
    gdouble lift5_height;
    gint lift_switch;
    gdouble lift1_value;
    gdouble lift2_value;
    gdouble lift3_value;
    gdouble lift4_value;
    gdouble lift5_value;

    /*litho data*/
    gint litho_routing;
   
    /*lua data*/
    gint lua_size;
 
    /*channels for preview*/
    gint nchan;
//    guchar descriptions[50][50];
    gchar **descriptions;
    /*preview data*/
    GwyDataField **preview_df;
    gdouble **preview_data;

    /*widgets*/
    GtkWidget *toplevel;
    GtkWidget *statusbar_measure;
    GtkWidget *statusbar_scan;
  
    GtkTextBuffer *tblog;
    GtkWidget *viewlog;

    GtkWidget *if_panel;

    GwsChanview **view;
    GwsChangraph **graph;

    GtkWidget *mode_combo;
    GtkWidget *rampmode_combo;
    GtkWidget *liftmode_combo;
    GtkWidget *liftswitch_combo;
    GtkWidget *button_adjust;
    GtkWidget *button_ramp;

    GtkWidget *out1rule_combo;
    GtkWidget *out2rule_combo;

    GtkWidget *kpfm_routing_combo;

    GtkObject *lua_size_adj;
    GtkTextBuffer *luabuffer;
    GtkWidget *luaview;
    GtkWidget *lua_button_load;
    GtkWidget *lua_button_save;

    GtkTextBuffer *lithobuffer;
    GtkWidget *lithoview;
    GtkWidget *litho_routing_combo;
    GtkWidget *litho_button_run;
    GtkWidget *litho_button_load;
    GtkWidget *litho_button_save;

    GtkWidget *fbunit_label;
    GtkWidget *fbunitc_label;
    GtkWidget *fmpid_label;
    GtkWidget *fm_label;
    GtkWidget *fmsetpoint_label;
    GtkWidget *fmunit_label;
    GtkWidget *fmunitc_label;
    GtkWidget *rampq_label;
    GtkWidget *rampqc_label;
    GtkWidget *lift1_label;
    GtkWidget *lift2_label;
    GtkWidget *lift3_label;
    GtkWidget *lift4_label;
    GtkWidget *lift5_label;

    /*adjustments holding a value*/
    GtkObject *threshold_adj;
    GtkObject *xres_adj;
    GtkObject *yres_adj;
    GtkObject *xreal_adj;
    GtkObject *yreal_adj;
    GtkObject *xoff_adj;
    GtkObject *yoff_adj;
    GtkObject *profile_adj;
    GtkObject *rotate_adj;
    GtkObject *rxres_adj;
    GtkObject *ryres_adj;
    GtkObject *rxreal_adj;
    GtkObject *ryreal_adj;
    GtkObject *rxoff_adj;
    GtkObject *ryoff_adj;
    GtkObject *rprofile_adj;
    GtkObject *rrotate_adj;
    GtkObject *wait_adj;
    GtkObject *speed_adj;
    GtkObject *nvals_adj;
    GtkObject *nprofiles_adj;
    GtkObject *noctaves_adj;
    GtkObject *order_adj;
    GtkObject *preview_step_adj;
    GtkObject *pidp_adj;
    GtkObject *pidi_adj;
    GtkObject *pidd_adj;
    GtkObject *fmpidp_adj;
    GtkObject *fmpidi_adj;
    GtkObject *fmpidd_adj;
    GtkObject *fmsetpoint_adj;
    GtkObject *zpiezo_adj;

    GtkObject *tipv_adj;
    GtkObject *samplev_adj;
    GtkObject *out1value_adj;
    GtkObject *out2value_adj;

    GtkObject *ramp_zres_adj;
    GtkObject *ramp_zfrom_adj;
    GtkObject *ramp_zto_adj;
    GtkObject *ramp_time_adj;
    GtkObject *ramp_starttime_adj;
    GtkObject *ramp_peaktime_adj;
    GtkObject *ramp_xpos_adj;
    GtkObject *ramp_ypos_adj;

    GtkObject *nlift_adj;
    GtkObject *lift1_height_adj;
    GtkObject *lift2_height_adj;
    GtkObject *lift3_height_adj;
    GtkObject *lift4_height_adj;
    GtkObject *lift5_height_adj;
    GtkObject *lift1_value_adj;
    GtkObject *lift2_value_adj;
    GtkObject *lift3_value_adj;
    GtkObject *lift4_value_adj;
    GtkObject *lift5_value_adj;

    GtkObject *kpfm_frequency_adj;
    GtkObject *kpfm_amplitude_adj;
    GtkObject *kpfm_offset_adj;
    GtkObject *kpfm_phase_adj;
    GtkObject *kpfm_bw_adj;
  
    /*spinbuttons - widgets for adjustments*/
    GtkWidget *xres_spin;
    GtkWidget *yres_spin;
    GtkWidget *xreal_spin;
    GtkWidget *yreal_spin;
    GtkWidget *xoff_spin;
    GtkWidget *yoff_spin;
    GtkWidget *profile_spin;
    GtkWidget *rotate_spin;
    GtkWidget *rxres_spin;
    GtkWidget *ryres_spin;
    GtkWidget *rxreal_spin;
    GtkWidget *ryreal_spin;
    GtkWidget *rxoff_spin;
    GtkWidget *ryoff_spin;
    GtkWidget *rprofile_spin;
    GtkWidget *rrotate_spin;
    GtkWidget *wait_spin;
    GtkWidget *speed_spin;
    GtkWidget *nvals_spin;
    GtkWidget *nprofiles_spin;
    GtkWidget *noctaves_spin;
    GtkWidget *order_spin;
    GtkWidget *preview_step_spin;
    GtkWidget *pidp_spin;
    GtkWidget *pidi_spin;
    GtkWidget *pidd_spin;
    GtkWidget *fmpidp_spin;
    GtkWidget *fmpidi_spin;
    GtkWidget *fmpidd_spin;
    GtkWidget *fmsetpoint_spin;
    GtkWidget *zpiezo_spin;
    GtkWidget *phasefilter_check;
    GtkWidget *amfm_check;
    GtkWidget *fmlimit_combo;
    GtkWidget *phaselimit_combo;

    GtkWidget *tipv_spin;
    GtkWidget *samplev_spin;
    GtkWidget *out1value_spin;
    GtkWidget *out2value_spin;

    GtkWidget *ramp_zres_spin;
    GtkWidget *ramp_zfrom_spin;
    GtkWidget *ramp_zto_spin;
    GtkWidget *ramp_time_spin;
    GtkWidget *ramp_starttime_spin;
    GtkWidget *ramp_peaktime_spin;
    GtkWidget *ramp_xpos_spin;
    GtkWidget *ramp_ypos_spin;

    GtkWidget *nlift_spin;
    GtkWidget *lift1_height_spin;
    GtkWidget *lift2_height_spin;
    GtkWidget *lift3_height_spin;
    GtkWidget *lift4_height_spin;
    GtkWidget *lift5_height_spin;
    GtkWidget *lift1_value_spin;
    GtkWidget *lift2_value_spin;
    GtkWidget *lift3_value_spin;
    GtkWidget *lift4_value_spin;
    GtkWidget *lift5_value_spin;

    GtkWidget *kpfm_frequency_spin;
    GtkWidget *kpfm_amplitude_spin;
    GtkWidget *kpfm_offset_spin;
    GtkWidget *kpfm_phase_spin;
    GtkWidget *kpfm_bw_spin;

    GtkWidget *lua_size_spin;


    /*labels to be hidden and shown on demand*/
    GtkWidget *xres_label;
    GtkWidget *yres_label;
    GtkWidget *nvals_label;
    GtkWidget *noctaves_label;
    GtkWidget *nprofiles_label;
    GtkWidget *order_label;
    GtkWidget *rotate_label;

    GtkWidget *rxres_label;
    GtkWidget *ryres_label;
    GtkWidget *rrotate_label;
		    

    /*buttons*/
    GtkWidget *button_approach;
    GtkWidget *button_retract;
    GtkWidget *button_tipup;
    GtkWidget *button_tipdown;
    GtkWidget *button_hwresettime;
    GtkWidget *button_scan;
    GtkWidget *button_pause;
    GtkWidget *button_createscanpath;

    GtkWidget *error_label;
    GtkWidget *fmerror_label;
    GtkWidget *zpiezo_label;
    GtkWidget *zpos_label;

    GtkWidget *zero_check;
    GtkWidget *freeze_check;
    GtkWidget *profile_check;
    GtkWidget *rprofile_check;

    GtkWidget *ramp_do_check;

    GtkWidget *path;

    ScanPath pathval;
    GtkWidget *pathlabel;

    gboolean running;

    /*scanning thread*/
    pthread_mutex_t *scan_mutex;
    pthread_mutex_t *ramp_mutex;
    pthread_mutex_t *dfield_mutex;
    pthread_mutex_t *rdfield_mutex;
    //GMutex* scan_mutex;
    //GMutex* ramp_mutex;
    //GMutex* dfield_mutex;
    //GMutex* rdfield_mutex;
    GThread *scan_loop;
    gboolean scanning;
    gint npos;          //allocated data for scanning
    gint rnpos;          //allocated data for scanning in reverse direction
    gint actual_profile;
    gint actual_profile_candidate;
    gdouble scan_start_time;
    gdouble scan_end_time;
    gint preview_step;    
   
    gdouble threshold;

    /*approaching thread*/
//    GMutex* approach_mutex;
    pthread_mutex_t *approach_mutex;
    GThread *approach_loop;
    gboolean approaching;

    /*creating path thread*/
//    GMutex* create_path_mutex;
    pthread_mutex_t *create_path_mutex;
    GThread *create_path_loop;
    gboolean creating_path;

    /*kill all threads, we are closing*/
    gboolean closingapp;

    GThread *hw_loop;


    /*bricks to play with force volume data*/
    GwyBrick *brick_z1;
    GwyBrick *brick_e1;
    GwyBrick *brick_i1;

    GwyBrick *brick_z2;
    GwyBrick *brick_e2;
    GwyBrick *brick_i2;


    /*realtime data, do not access it without mutex locks!*/

    /*values obtained from PI table*/
//    GMutex* cap_mutex;
    pthread_mutex_t *cap_mutex;
    gdouble xpos_actual;
    gdouble ypos_actual;

    /*values read from NI card*/
//    GMutex* zpiezo_mutex;
    pthread_mutex_t *zpiezo_mutex;
    gdouble zpiezo_actual;
    gdouble error_actual;
    gdouble in1_actual;
    gdouble in2_actual;
    gdouble time_actual;
    gdouble a1_actual;
    gdouble p1_actual;
    gdouble in_actual[16];

    /*synchronize socket*/
    //GMutex* sock_mutex;
    pthread_mutex_t *sock_mutex;

    /*hwserver start (or reset) time */
//    GMutex* hw_start_time_mutex;
    pthread_mutex_t *hw_start_time_mutex;
    gdouble hw_start_time;

    gpointer cda; //custom data a    
} MiniData;

gboolean init_data(MiniData *md, gchar* ini_folder, gint preview_size, void(*compar)(void*));
void free_data(MiniData *md);

void writelog(MiniData *md, gchar *message);
void writelog_date(MiniData *md);

void get_actual_pos(MiniData *md, gdouble *xpos, gdouble *ypos);
void get_actual_zdata(MiniData *md, gdouble *zpiezo, gdouble *error, gdouble *in1, gdouble *in2, gdouble *time, gdouble *a1, gdouble *p1);
void get_actual_in(MiniData *md, gdouble *in);
void get_actual_hw_start_time(MiniData *md, gdouble *time);
void get_actual_scan_times(MiniData *md, gdouble *start_time, gdouble *end_time);
void get_actual_whatever(MiniData *md, gdouble *val, gint channel);
void get_actual_preview_step(MiniData *md, gint *preview_step);

void update_preview_channels_data(MiniData *md);

void init_hw(MiniData *md);

void start_logging_and_threads(MiniData *md);

#endif

