/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include "freqview.h"
#include "data.h"
#include "hw.h"

#define MAXN 10000

static void gws_freqview_class_init          (GwsFreqViewClass *klass);
static void gws_freqview_init                (GwsFreqView      *ttt);

GType
gws_freqview_get_type (void)
{
  static GType freqview_type = 0;

  if (!freqview_type)
    {
      const GTypeInfo freqview_info =
      {
	sizeof (GwsFreqViewClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) gws_freqview_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (GwsFreqView),
	0,
	(GInstanceInitFunc) gws_freqview_init,
      };

      freqview_type = g_type_register_static (GTK_TYPE_TABLE, "GwsFreqView", &freqview_info, 0);
    }

  return freqview_type;
}

static void
gws_freqview_class_init (GwsFreqViewClass *klass)
{
}

static void
gws_freqview_changed_cb(GtkObject *object, GwsFreqView* freqview)
{
    freqview->from = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->from_adj));
    freqview->to = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->to_adj));
    freqview->step = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->step_adj));
    freqview->exc1 = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->exc1_adj));
    freqview->exc2 = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->exc2_adj));
    freqview->delay = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->delay_adj));
    freqview->freq = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->freq_adj));
   // freqview->aout1 = gtk_adjustment_get_value(GTK_ADJUSTMENT(freqview->aout1_adj));

//    set_analog_value((MiniData *)freqview->md, 1, freqview->aout1);
}

static void
acquire_cb(GtkWidget *widget, GwsFreqView* freqview)
{
    MiniData *md = (MiniData *)freqview->md;
    gdouble f, p, a, buf;
    gint n;
   

    if (freqview->step<=0) freqview->step = 1;    

    printf("scanning: from %g to %g step %g kHz\n", freqview->from, freqview->to, freqview->step); 

    n = 0;
    set_freq(md, 1, freqview->from*1000, freqview->exc1, 0); //from kHz to Hz
    set_freq(md, 2, freqview->from*1000, freqview->exc2, 0); //from kHz to Hz

    usleep(100000);

    for (f=freqview->from; f<freqview->to; f+=freqview->step) 
    {
       slide_freq(md, 1, f*1000);
       slide_freq(md, 2, f*1000);

       usleep(freqview->delay*1000);

       get_actual_zdata(md, &buf, &buf, &buf, &buf, &buf, &a, &p);
       freqview->fdata[n] = f;          //in kHz already
       freqview->adata[n] = a;          //in V already
       freqview->pdata[n] = p;//*180/M_PI; //in deg

       n++;
    }
  /*  for (f=freqview->to; f>=freqview->from; f-=freqview->step) 
    {
       set_freq(md, 1, f*1000, freqview->exc1, 0); //from kHz to Hz
       set_freq(md, 2, f*1000, freqview->exc2, 0); //from kHz to Hz

       usleep(freqview->delay*1000);

       get_actual_zdata(md, &buf, &a, &buf, &p, &buf);
       freqview->fdata[n] = f;          //in kHz already
       freqview->adata[n] = a;          //in V already
       freqview->pdata[n] = p*180/M_PI; //in deg

       n++;
    }*/
    freqview->ndata = n;

    gwy_graph_curve_model_set_data(freqview->gcmodela, freqview->fdata, freqview->adata, freqview->ndata);
    gwy_graph_curve_model_set_data(freqview->gcmodelp, freqview->fdata, freqview->pdata, freqview->ndata);

}

void
gws_freqview_set_inits(GwsFreqView *freqview, gdouble from, gdouble to, gdouble step, gdouble exc1, gdouble exc2, gint delay, gdouble freq, gdouble aout1, gint phaseshift)
{
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->from_adj), from);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->to_adj), to);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->step_adj), step);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->exc1_adj), exc1);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->exc2_adj), exc2);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->delay_adj), delay);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->freq_adj), freq);

//    gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->aout1_adj), aout1);

    gwy_enum_combo_box_set_active(GTK_COMBO_BOX(freqview->shift_combo), phaseshift);
}

gdouble
gws_freqview_get_freq(GwsFreqView *freqview)
{
    return freqview->freq;
}

static void
agraph_selected(GwySelection *selection, gint hint, GwsFreqView *freqview)
{
    gint nselections;
    gdouble sel[1], freq;
    MiniData *md = (MiniData *)freqview->md;

    g_return_if_fail(hint <= 0);

    nselections = gwy_selection_get_data(selection, NULL);
    gwy_selection_get_object(selection, 0, sel);

    if (nselections>0) 
    {
        freq = sel[0];
        gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->freq_adj), freq);

        set_freq(md, 1, freq*1000, freqview->exc1, 0); //from kHz to Hz
        set_freq(md, 2, freq*1000, freqview->exc2, 0); //from kHz to Hz


        gwy_selection_clear(gwy_graph_area_get_selection(GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(freqview->graphp))), GWY_GRAPH_STATUS_XLINES));
    }
}

static void
pgraph_selected(GwySelection *selection, gint hint, GwsFreqView *freqview)
{
    gint nselections;
    gdouble sel[1], freq;
    MiniData *md = (MiniData *)freqview->md;

    g_return_if_fail(hint <= 0);

    nselections = gwy_selection_get_data(selection, NULL);
    gwy_selection_get_object(selection, 0, sel);

    if (nselections>0) 
    {
        freq = sel[0];
        gtk_adjustment_set_value(GTK_ADJUSTMENT(freqview->freq_adj), freq);

        set_freq(md, 1, freq*1000, freqview->exc1, 0); //from kHz to Hz
        set_freq(md, 2, freq*1000, freqview->exc2, 0); //from kHz to Hz

        gwy_selection_clear(gwy_graph_area_get_selection(GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(freqview->grapha))), GWY_GRAPH_STATUS_XLINES));
    }
}

static void
shift_changed_cb(GtkComboBox *combo, GwsFreqView *freqview)
{
    MiniData *md = (MiniData *)freqview->md;

    freqview->shift = gwy_enum_combo_box_get_active(combo);
    printf("shift set to %d\n", freqview->shift);

    set_phaseshift(md, freqview->shift);
}


static void
gws_freqview_init (GwsFreqView *freqview)
{
  GtkWidget *label, *button;
  GwySelection *selection;
  guint row = 0;

   static const GwyEnum shifts[] = {
        { N_("0 deg"),           0, },
        { N_("90 deg"),          1, },
        { N_("180 deg"),         2, },
        { N_("270 deg"),         3, },
   };


  freqview->ndata = MAXN;
  freqview->fdata = (gdouble *)g_malloc(freqview->ndata*sizeof(gdouble));
  freqview->adata = (gdouble *)g_malloc(freqview->ndata*sizeof(gdouble));
  freqview->pdata = (gdouble *)g_malloc(freqview->ndata*sizeof(gdouble));


  gtk_table_resize (GTK_TABLE(freqview), 4, 4);

  freqview->gcmodela = gwy_graph_curve_model_new();
  freqview->gmodela = gwy_graph_model_new();
  g_object_set(freqview->gmodela,
                 "axis-label-bottom", "frequency [kHz]",
                 "axis-label-left", "amplitude [V]",
                 NULL);
  g_object_set(freqview->gcmodela,
                 "mode", GWY_GRAPH_CURVE_LINE,
                 NULL);
  gwy_graph_model_add_curve(freqview->gmodela, freqview->gcmodela);
  freqview->grapha = GWY_GRAPH(gwy_graph_new(freqview->gmodela));

  gwy_graph_set_status(GWY_GRAPH(freqview->grapha), GWY_GRAPH_STATUS_XLINES);
  gwy_graph_enable_user_input(GWY_GRAPH(freqview->grapha), FALSE);
  selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(freqview->grapha))), GWY_GRAPH_STATUS_XLINES);
  gwy_selection_set_max_objects(selection, 1);
  g_signal_connect(selection, "changed", G_CALLBACK(agraph_selected), freqview);

  gtk_widget_set_size_request(GTK_WIDGET(freqview->grapha), 600, 200);

  gtk_table_attach(GTK_TABLE(freqview), GTK_WIDGET(freqview->grapha),
                   0, 4, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  freqview->gcmodelp = gwy_graph_curve_model_new();
  freqview->gmodelp = gwy_graph_model_new();
  g_object_set(freqview->gmodelp,
                 "axis-label-bottom", "frequency [kHz]",
                 "axis-label-left", "phase [rad]",
                 NULL);
   g_object_set(freqview->gcmodelp,
                 "mode", GWY_GRAPH_CURVE_LINE,
                 NULL);
  gwy_graph_model_add_curve(freqview->gmodelp, freqview->gcmodelp);
  freqview->graphp = GWY_GRAPH(gwy_graph_new(freqview->gmodelp));

  gwy_graph_set_status(GWY_GRAPH(freqview->graphp), GWY_GRAPH_STATUS_XLINES);
  gwy_graph_enable_user_input(GWY_GRAPH(freqview->graphp), FALSE);
  selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(freqview->graphp))), GWY_GRAPH_STATUS_XLINES);
  gwy_selection_set_max_objects(selection, 1);
  g_signal_connect(selection, "changed", G_CALLBACK(pgraph_selected), freqview);

  gtk_widget_set_size_request(GTK_WIDGET(freqview->graphp), 600, 200);

  gtk_table_attach(GTK_TABLE(freqview), GTK_WIDGET(freqview->graphp),
                   0, 4, row, row+1, GTK_FILL, 0, 5, 1);

  row++;


  label = gtk_label_new("Range:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->from_adj = gtk_adjustment_new(freqview->from, 1, 1000, 0.01, 1, 0);
  freqview->from_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->from_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->from_spin), 2);
  g_signal_connect(freqview->from_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->from_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  freqview->to_adj = gtk_adjustment_new(freqview->to, 1, 1000, 0.01, 1, 0);
  freqview->to_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->to_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->to_spin), 2);
  g_signal_connect(freqview->to_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->to_spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("kHz");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   3, 4, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Step:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->step_adj = gtk_adjustment_new(freqview->step, 0.001, 10, 0.001, 0.1, 0);
  freqview->step_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->step_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->step_spin), 3);
  g_signal_connect(freqview->step_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->step_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("kHz");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Excitation 1:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->exc1_adj = gtk_adjustment_new(freqview->exc1, 0, 1, 0.0001, 0.001, 0);
  freqview->exc1_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->exc1_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->exc1_spin), 4);
  g_signal_connect(freqview->exc1_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->exc1_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("V");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Excitation 2:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->exc2_adj = gtk_adjustment_new(freqview->exc2, 0, 1, 0.0001, 0.001, 0);
  freqview->exc2_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->exc2_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->exc2_spin), 4);
  g_signal_connect(freqview->exc2_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->exc2_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("V");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Settle:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->delay_adj = gtk_adjustment_new(freqview->delay, 0, 1000, 1, 10, 0);
  freqview->delay_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->delay_adj), 1, 2);
  g_signal_connect(freqview->delay_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->delay_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("ms");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Frequency set:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->freq_adj = gtk_adjustment_new(freqview->freq, 1, 1000, 1, 10, 0);
  freqview->freq_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->freq_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->freq_spin), 4);
  g_signal_connect(freqview->freq_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->freq_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("kHz");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;

  label = gtk_label_new("Phase shift:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

  freqview->shift_combo
        = gwy_enum_combo_box_new(shifts, G_N_ELEMENTS(shifts),
                                 G_CALLBACK(shift_changed_cb),
                                 freqview, freqview->shift, TRUE);
  gtk_table_attach(GTK_TABLE(freqview), freqview->shift_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

   row++;

/*
  label = gtk_label_new("Analog output 1:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


  freqview->aout1_adj = gtk_adjustment_new(freqview->aout1, 0, 1.8, 0.001, 0.01, 0);
  freqview->aout1_spin = gtk_spin_button_new(GTK_ADJUSTMENT(freqview->aout1_adj), 1, 2);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(freqview->aout1_spin), 3);
  g_signal_connect(freqview->aout1_adj, "value-changed",
                     G_CALLBACK(gws_freqview_changed_cb), freqview);
  gtk_table_attach(GTK_TABLE(freqview), freqview->aout1_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

  label = gtk_label_new("V");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(freqview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

  row++;
*/

  button = gtk_button_new_with_label("Acquire");
  gtk_table_attach(GTK_TABLE(freqview), button,
                   1, 3, row, row+1, GTK_FILL, 0, 5, 1);

  g_signal_connect(button, "clicked",
                             G_CALLBACK(acquire_cb), freqview);

  gtk_widget_show_all(GTK_WIDGET(freqview));


}

GtkWidget*
gws_freqview_new(gpointer md)
{
  GwsFreqView *fv = g_object_new (gws_freqview_get_type (), NULL);
  fv->md = md;

  return GTK_WIDGET(fv);
}


