/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __FREQVIEW_H__
#define __FREQVIEW_H__


#include <glib.h>
#include <glib-object.h>
#include <gtk/gtktable.h>


G_BEGIN_DECLS

#define GWS_FREQVIEW_TYPE            (gws_freqview_get_type ())
#define GWS_FREQVIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GWS_FREQVIEW_TYPE, GwsFreqView))
#define GWS_FREQVIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GWS_FREQVIEW_TYPE, GwsFreqViewClass))
#define GWS_IS_FREQVIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GWS_FREQVIEW_TYPE))
#define GWS_IS_FREQVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GWS_FREQVIEW_TYPE))


typedef struct _GwsFreqView       GwsFreqView;
typedef struct _GwsFreqViewClass  GwsFreqViewClass;

struct _GwsFreqView
{
    GtkTable table;

    GThread *update_loop;

    gdouble from;
    gdouble to;
    gdouble step;
    gdouble exc1;
    gdouble exc2;
    gint delay;
    gdouble freq;
    gdouble aout1;
    gint shift;

    GtkObject *from_adj;
    GtkObject *to_adj;
    GtkWidget *from_spin;
    GtkWidget *to_spin;
    GtkObject *step_adj;
    GtkWidget *step_spin;
    GtkObject *exc1_adj;
    GtkWidget *exc1_spin;
    GtkObject *exc2_adj;
    GtkWidget *exc2_spin;

    GtkObject *aout1_adj;
    GtkWidget *aout1_spin;

    GtkWidget *shift_combo;
 
    GtkObject *delay_adj;
    GtkWidget *delay_spin;

    GtkObject *freq_adj;
    GtkWidget *freq_spin;

    gdouble *fdata;
    gdouble *adata;
    gdouble *pdata;
    gint ndata;

    GwyGraphCurveModel *gcmodela;
    GwyGraphModel *gmodela;
    GwyGraph *grapha;

    GwyGraphCurveModel *gcmodelp;
    GwyGraphModel *gmodelp;
    GwyGraph *graphp;

    gpointer md;

};

struct _GwsFreqViewClass
{
  GtkTableClass parent_class;

  void (* freqview) (GwsFreqView *freqview);
};

GType          gws_freqview_get_type        (void);
GtkWidget*     gws_freqview_new             (gpointer md);

void           gws_freqview_set_inits       (GwsFreqView *freqview, gdouble from, gdouble to, gdouble step, gdouble exc1, gdouble exc2, gint delay, gdouble freq, gdouble aout1, gint phaseshift);

gdouble        gws_freqview_get_freq        (GwsFreqView *freqview);


G_END_DECLS

#endif /* __FREQVIEW_H__ */
