
/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
*/

#include <stdio.h>
#include "hw.h"
#include "gui.h"

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "gwyfile.h"
#include "clientserver.h"

#define HWDEBUG 0


GwyfileObject* send_gwy_message(gint sock, GwyfileObject *gwyf)
{
    GwyfileObject *response;
    int err;
    char *buffer = NULL;
    size_t bufsize = 0;
    //GwyfileItem *item;
    //unsigned int nitems, i;

    if (send_gwyfile_message(gwyf, sock, &buffer, &bufsize) < 0) 
    {
        fprintf(stderr, "Cannot send message\n");
        return NULL;
    }
    //gwyfile_object_free(gwyf);
    //printf("Sent message, waiting for response.\n");

    response = recv_gwyfile_message(sock, &buffer, &bufsize, &err);
    if (err < 0) {
        fprintf(stderr, "Cannot receive message\n");
        return NULL;
    }

    /*nitems = gwyfile_object_nitems(response);
    printf("Response has %u items\n", nitems);
    for (i = 0; i < nitems; i++) {
        item = gwyfile_object_get_nth(response, i);
        printf("Item[%u] is called \"%s\" (type %c).\n", i, gwyfile_item_name(item), gwyfile_item_type(item));
    }
    gwyfile_object_free(response);*/

    g_free(buffer);

    return response;
}



gboolean get_values(gint sock, pthread_mutex_t* mutex,
                    gdouble *xpos, gdouble *ypos, gdouble *zpiezo, 
                    gdouble *errorsignal, gdouble *in1, gdouble *in2, gdouble *timestamp, gdouble *a1, gdouble *p1, gdouble *in, gint *lost)
{    

    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;
    const char *todo;
    gint i;
    gchar key[20];

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "read"),
                           NULL);

    pthread_mutex_lock(mutex);
    response = send_gwy_message(sock, gwyf);
    pthread_mutex_unlock(mutex);
  
    if (response==NULL) {
       *lost = 1;
       return FALSE;
    }
 
    gwyfile_object_free(gwyf);

    item = gwyfile_object_get(response, "todo");

    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "read") == 0) {
           valueitem = gwyfile_object_get(response, "x");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain x\n");
           else {
               *xpos = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "y");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain y\n");
           else {
               *ypos = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "z");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain z\n");
           else {
               *zpiezo = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "e");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain e\n");
           else {
               *errorsignal = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "a1");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain a1\n");
           else {
               *a1 = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "p1");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain p1\n");
           else {
               *p1 = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "adc1");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain adc1\n");
           else {
               *in1 = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "adc2");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain adc2\n");
           else {
               *in2 = gwyfile_item_get_double(valueitem);
           }
           valueitem = gwyfile_object_get(response, "ts");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain ts\n");
           else {
               *timestamp = gwyfile_item_get_double(valueitem);
           }
           for (i=0; i<16; i++) 
           {
               snprintf(key, sizeof(key), "in%d", i+1);
               valueitem = gwyfile_object_get(response, key);
               if (!valueitem) {
                     //printf("no in %d\n", i);
                     in[i] = 0;
               }
               else in[i] = gwyfile_item_get_double(valueitem);

           }
 
    } else printf("wrong response, todo should be read, it is %s\n", todo);

    //printf("a %g p %g   errorsignal %g\n", *a1, *p1, *errorsignal);

    gwyfile_object_free(response);


    return TRUE;

}


gboolean set_motor_up(MiniData *md)
{
    gboolean result = TRUE;
    printf("mup!\n");
    table_move_to(md, 1.0e-6, 0.0);
    return result;
}

gboolean set_motor_down(MiniData *md)
{
    gboolean result = TRUE;
    printf("mdown!\n");
    table_move_to(md, 0.0, 0.0);
    return result;
}


gboolean set_standby_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd)
{
    GwyfileObject *gwyf, *response;

    printf("setting standby storage, a2 %d p2 %d,  ins: %d%d%d%d %d%d%d%d %d%d%d%d %d%d%d%d %d\n", a2, p2, 
                                      in[0], in[1], in[2], in[3],
                                      in[4], in[5], in[6], in[7],
                                      in[8], in[9], in[10], in[11],
                                      in[12], in[13], in[14], in[15], fmd);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_standby_storage"),
                           gwyfile_item_new_bool("a1", TRUE),
                           gwyfile_item_new_bool("p1", TRUE),
                           gwyfile_item_new_bool("a2", a2),
                           gwyfile_item_new_bool("p2", p2),
                           gwyfile_item_new_bool("in1", in[0]),
                           gwyfile_item_new_bool("in2", in[1]),
                           gwyfile_item_new_bool("in3", in[2]),
                           gwyfile_item_new_bool("in4", in[3]),
                           gwyfile_item_new_bool("in5", in[4]),
                           gwyfile_item_new_bool("in6", in[5]),
                           gwyfile_item_new_bool("in7", in[6]),
                           gwyfile_item_new_bool("in8", in[7]),
                           gwyfile_item_new_bool("in9", in[8]),
                           gwyfile_item_new_bool("in10", in[9]),
                           gwyfile_item_new_bool("in11", in[10]),
                           gwyfile_item_new_bool("in12", in[11]),
                           gwyfile_item_new_bool("in13", in[12]),
                           gwyfile_item_new_bool("in14", in[13]),
                           gwyfile_item_new_bool("in15", in[14]),
                           gwyfile_item_new_bool("in16", in[15]),
                           gwyfile_item_new_bool("fmdrive", fmd),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}
gboolean set_scan_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd)
{
    GwyfileObject *gwyf, *response;

    printf("setting scan storage, a2 %d p2 %d,  ins: %d%d%d%d %d%d%d%d %d%d%d%d %d%d%d%d %d\n", a2, p2, 
                                      in[0], in[1], in[2], in[3],
                                      in[4], in[5], in[6], in[7],
                                      in[8], in[9], in[10], in[11],
                                      in[12], in[13], in[14], in[15], fmd);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_scan_storage"),
                           gwyfile_item_new_bool("a1", TRUE),
                           gwyfile_item_new_bool("p1", TRUE),
                           gwyfile_item_new_bool("a2", a2),
                           gwyfile_item_new_bool("p2", p2),
                           gwyfile_item_new_bool("in1", in[0]),
                           gwyfile_item_new_bool("in2", in[1]),
                           gwyfile_item_new_bool("in3", in[2]),
                           gwyfile_item_new_bool("in4", in[3]),
                           gwyfile_item_new_bool("in5", in[4]),
                           gwyfile_item_new_bool("in6", in[5]),
                           gwyfile_item_new_bool("in7", in[6]),
                           gwyfile_item_new_bool("in8", in[7]),
                           gwyfile_item_new_bool("in9", in[8]),
                           gwyfile_item_new_bool("in10", in[9]),
                           gwyfile_item_new_bool("in11", in[10]),
                           gwyfile_item_new_bool("in12", in[11]),
                           gwyfile_item_new_bool("in13", in[12]),
                           gwyfile_item_new_bool("in14", in[13]),
                           gwyfile_item_new_bool("in15", in[14]),
                           gwyfile_item_new_bool("in16", in[15]),
                           gwyfile_item_new_bool("fmdrive", fmd),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_ramp_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd)
{
    GwyfileObject *gwyf, *response;

    printf("setting ramp storage, a2 %d p2 %d,  ins: %d%d%d%d %d%d%d%d %d%d%d%d %d%d%d%d %d\n", a2, p2, 
                                      in[0], in[1], in[2], in[3],
                                      in[4], in[5], in[6], in[7],
                                      in[8], in[9], in[10], in[11],
                                      in[12], in[13], in[14], in[15], fmd);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_ramp_storage"),
                           gwyfile_item_new_bool("a1", TRUE),
                           gwyfile_item_new_bool("p1", TRUE),
                           gwyfile_item_new_bool("a2", a2),
                           gwyfile_item_new_bool("p2", p2),
                           gwyfile_item_new_bool("in1", in[0]),
                           gwyfile_item_new_bool("in2", in[1]),
                           gwyfile_item_new_bool("in3", in[2]),
                           gwyfile_item_new_bool("in4", in[3]),
                           gwyfile_item_new_bool("in5", in[4]),
                           gwyfile_item_new_bool("in6", in[5]),
                           gwyfile_item_new_bool("in7", in[6]),
                           gwyfile_item_new_bool("in8", in[7]),
                           gwyfile_item_new_bool("in9", in[8]),
                           gwyfile_item_new_bool("in10", in[9]),
                           gwyfile_item_new_bool("in11", in[10]),
                           gwyfile_item_new_bool("in12", in[11]),
                           gwyfile_item_new_bool("in13", in[12]),
                           gwyfile_item_new_bool("in14", in[13]),
                           gwyfile_item_new_bool("in15", in[14]),
                           gwyfile_item_new_bool("in16", in[15]),
                           gwyfile_item_new_bool("fmdrive", fmd),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}


gboolean table_move_to(MiniData *md, gdouble xpos, gdouble ypos)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "move_to"),
                           gwyfile_item_new_double("xreq", xpos),
                           gwyfile_item_new_double("yreq", ypos),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_analog_value(MiniData *md, gint channel, gdouble val)
{
    GwyfileObject *gwyf, *response;
    gchar key[50];

    snprintf(key, sizeof(key), "out%d", channel+1);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_out"),
                           gwyfile_item_new_double(key, val),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);
    return TRUE;
}

gboolean set_filter(MiniData *md, gint channel, gint val)
{
    GwyfileObject *gwyf, *response;

    if (channel == 0)
       gwyf = gwyfile_object_new("GS",
                              gwyfile_item_new_string_copy("todo", "set"),
                              gwyfile_item_new_int32("filter1", val),
                              NULL);
    else
       gwyf = gwyfile_object_new("GS",
                              gwyfile_item_new_string_copy("todo", "set"),
                              gwyfile_item_new_int32("filter2", val),
                              NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);
    return TRUE;
}

static
const gchar* rule_to_string(int rule)
{
    if (rule==GWS_RULE_X) return "x";
    else if (rule==GWS_RULE_Y) return "y";
    else if (rule==GWS_RULE_Z) return "y";
    else if (rule==GWS_RULE_IN1) return "in1";
    else if (rule==GWS_RULE_IN2) return "in2";

    else return "nothing";
}


gboolean set_routing(MiniData *md, const gchar *channel, gint rule)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                        gwyfile_item_new_string_copy("todo", "set_routing"),
                        gwyfile_item_new_string_copy(channel, rule_to_string(rule)),
                        NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

gboolean set_speed(MiniData *md, gdouble val)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_scan"),
                           gwyfile_item_new_double("speed", val/1.0e6), //entered in um/s converted to m/s
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_pid(MiniData *md, gdouble pidp, gdouble pidi, gdouble pidd)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("pid1_p", pidp), 
                           gwyfile_item_new_double("pid1_i", pidi),
                           gwyfile_item_new_double("pid1_d", pidd),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_fmpid(MiniData *md, gdouble pidp, gdouble pidi, gdouble pidd)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("pid2_p", pidp), 
                           gwyfile_item_new_double("pid2_i", pidi),
                           gwyfile_item_new_double("pid2_d", pidd),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    printf("set fmpid to %g %g %g\n", pidp, pidi, pidd);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_fmlimit(MiniData *md, gint fmlimit)
{
    GwyfileObject *gwyf, *response;

    printf("set fmlimit to %d\n", fmlimit);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_int32("fm1_frequency_limit_factor", fmlimit), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_phaselimit(MiniData *md, gint phaselimit)
{
    GwyfileObject *gwyf, *response;

    printf("set phaselimit to %d\n", phaselimit);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_int32("fm1_phase_limit_factor", phaselimit), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}
gboolean set_filterphase(MiniData *md, gboolean filterphase)
{
    GwyfileObject *gwyf, *response;
  
    printf("set filterphase to %d\n", filterphase);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_int32("fm1_filter_phase", filterphase), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_setpoint(MiniData *md, gdouble setpoint)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("pid1_setpoint", setpoint), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_fmsetpoint(MiniData *md, gdouble setpoint)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("fm1_setpoint", setpoint), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_fmfeedback(MiniData *md, gboolean feedback)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_feedback"),
                           gwyfile_item_new_bool("fmfeedback1", feedback), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_fmam_feedback(MiniData *md, gboolean feedback)
{
    GwyfileObject *gwyf, *response;

    printf("set fmam to %d\n", feedback);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_feedback"),
                           gwyfile_item_new_bool("fmfeedback2", feedback), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}
gboolean set_phaseshift(MiniData *md, gint phaseshift)
{
    GwyfileObject *gwyf, *response;

    printf("set phaseshift to %d\n", phaseshift);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_int32("phaseshift1", phaseshift), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_feedback(MiniData *md, gboolean feedback)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_feedback"),
                           gwyfile_item_new_bool("feedback1", feedback), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean set_zpiezo(MiniData *md, gdouble zpiezo)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_feedback"),
                           gwyfile_item_new_double("zpiezo", zpiezo), 
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

gboolean set_freq(MiniData *md, gint channel, gdouble frequency, gdouble amplitude, gdouble phase)
{
    GwyfileObject *gwyf, *response;
    if (channel==1)
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("freq1_f", frequency), 
                           gwyfile_item_new_double("freq1_a", amplitude),
                           gwyfile_item_new_double("freq1_p", phase),
                           NULL);
    else 
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("freq2_f", frequency), 
                           gwyfile_item_new_double("freq2_a", amplitude),
                           gwyfile_item_new_double("freq2_p", phase),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

gboolean slide_freq(MiniData *md, gint channel, gdouble frequency)
{
    GwyfileObject *gwyf, *response;
    if (channel==1)
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("freq1_f", frequency),
                           NULL);
    else
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("freq2_f", frequency),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

gboolean set_wait(MiniData *md, gdouble val)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_scan"),
                           gwyfile_item_new_double("delay", val),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) return FALSE;
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}


gboolean get_ramp_data(MiniData *md, gint *res, gdouble *zdata, gdouble *edata, gdouble *idata)
{
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;
    const char *todo;
    const gdouble *vals;
    gint i;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "read_ramp"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "read_ramp") == 0) {

           valueitem = gwyfile_object_get(response, "zres");
           if (!valueitem) fprintf(stderr, "Error! read_ramp message does not contain zres\n");
           else {
               *res = gwyfile_item_get_int32(valueitem);
           }

           valueitem = gwyfile_object_get(response, "zdata");
           if (!valueitem) fprintf(stderr, "Error! read_ramp message does not contain zdata\n");
           else {
               vals = gwyfile_item_get_double_array(valueitem);
               for (i=0; i<1024; i++) zdata[i] = vals[i]; 
           }

           valueitem = gwyfile_object_get(response, "edata");
           if (!valueitem) fprintf(stderr, "Error! read_ramp message does not contain edata\n");
           else {
               vals = gwyfile_item_get_double_array(valueitem);
               for (i=0; i<1024; i++) edata[i] = vals[i]; 
           }

           valueitem = gwyfile_object_get(response, "idata");
           if (!valueitem) fprintf(stderr, "Error! read_ramp message does not contain idata\n");
           else {
               vals = gwyfile_item_get_double_array(valueitem);
               for (i=0; i<1024; i++) idata[i] = vals[i]; 
           }
    } else printf("Error, expected read_ramp result, got something else (%s)\n", todo);

    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return TRUE; 
}

gint get_scan_ndata(MiniData *md)
{    
    gint n = -1;
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;
    const char *todo;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get_scan_ndata"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "get_scan_ndata") == 0) {
           valueitem = gwyfile_object_get(response, "n");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain n\n");
           else {
               n = gwyfile_item_get_int32(valueitem);
           }
    }
    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return n;
}


gboolean set_mode(MiniData *md, gint mode)
{
    GwyfileObject *gwyf, *response;
    GwyfileItem *valueitem;

    md->running = FALSE;
    set_standby_storage(md, md->c_sc_a1, md->c_sc_p1, md->c_sc_in, FALSE); 

    printf("setting mode to %d\n", mode);
    if (mode==0)
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "off"),
                           NULL);
    else if (mode==1)
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "proportional"),
                           NULL);

    else if (mode==2)
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "ncamplitude"),
                           NULL);
    else if (mode==3) 
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "ncphase"),
                           NULL);
    else if (mode==4) 
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "akiyama"),
                           NULL);
    else 
       gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "state"),
                           gwyfile_item_new_string_copy("mode", "nenoprobe"),
                           NULL);



    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
    else {
       valueitem = gwyfile_object_get(response, "x_range");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain x_range\n");
       else md->xrange = gwyfile_item_get_double(valueitem);

       valueitem = gwyfile_object_get(response, "y_range");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain y_range\n");
       else md->yrange = gwyfile_item_get_double(valueitem);

       valueitem = gwyfile_object_get(response, "z_range");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain z_range\n");
       else md->zrange = gwyfile_item_get_double(valueitem);

       valueitem = gwyfile_object_get(response, "hrdac_range");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain hrdac_range\n");
       else md->hrdac_range = gwyfile_item_get_int32(valueitem);

       valueitem = gwyfile_object_get(response, "zpiezo_hrdac_range");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain zpiezo_hrdac_range\n");
       else md->zpiezo_hrdac_range = gwyfile_item_get_int32(valueitem);

       valueitem = gwyfile_object_get(response, "scan_mode");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain scan_mode\n");
       else md->xymode = gwyfile_item_get_int32(valueitem);

       valueitem = gwyfile_object_get(response, "zpiezo_hrdac");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain zpiezo_hrdac\n");
       else md->zpiezo_hrdac = gwyfile_item_get_bool(valueitem);

       valueitem = gwyfile_object_get(response, "swap_proportional");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain swap_proportional\n");
       else md->swap_proportional = gwyfile_item_get_bool(valueitem);

       valueitem = gwyfile_object_get(response, "swap_nenoprobe");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain swap_nenprobe\n");
       else md->swap_nenoprobe = gwyfile_item_get_bool(valueitem);

       valueitem = gwyfile_object_get(response, "swap_zpiezo");
       if (!valueitem) fprintf(stderr, "Error! read message does not contain swap_zpiezo\n");
       else md->swap_zpiezo = gwyfile_item_get_bool(valueitem);
    }

    md->running = TRUE;


    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return TRUE;
}

gboolean get_hw_start_time(MiniData *md, gdouble *hw_start_time)
{
    GwyfileObject *gwyf, *response;
    GwyfileItem *valueitem;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get"),
                           gwyfile_item_new_double("hwtime", 0.0),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);  ////
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    valueitem = gwyfile_object_get(response, "hwtime");
    if (!valueitem) fprintf(stderr, "Error! read message does not contain hwtime\n");
    else {
        *hw_start_time = gwyfile_item_get_double(valueitem);
    }

    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return TRUE;

}

gboolean get_scanning_script_status(MiniData *md)
{
    GwyfileObject *gwyf, *response;
    GwyfileItem *valueitem;
    gboolean result;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get"),
                           gwyfile_item_new_bool("scanning_script", FALSE),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    valueitem = gwyfile_object_get(response, "scanning_script");
    if (!valueitem) {
        fprintf(stderr, "Error! read message does not contain scanning_script\n");
        result = FALSE;
    }
    else {
        result = gwyfile_item_get_bool(valueitem);
    }

    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return result;
}

gboolean reset_start_time(MiniData *md)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set"),
                           gwyfile_item_new_double("hwtime", 0.0),
                           NULL);

    printf("reset start time will be issued on socket %d\n", md->sock);


    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

gboolean is_table_to(MiniData *md, gboolean *is_table_to)
{
    GwyfileObject *gwyf, *response;
    GwyfileItem *valueitem;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get"),
                           gwyfile_item_new_bool("moving", 0),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    valueitem = gwyfile_object_get(response, "moving");
    if (!valueitem) fprintf(stderr, "Error! read message does not contain moving\n");
    else {
        *is_table_to = gwyfile_item_get_bool(valueitem);
    }

    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return TRUE;
}


gpointer
hw_loop(gpointer data)
{
    MiniData *md = (MiniData *)data;
    gdouble xpos, ypos, zpiezo, timestamp, hw_start_time, a1, p1;
    gdouble errorsignal, in1, in2, in[16];
    gint i;
    struct sockaddr_in serv_addr;
    struct hostent *server;
    gint mode = 0;

    printf("starting hw loop, server %s port %d\n", md->server, md->port);


    md->sock = socket(AF_INET, SOCK_STREAM, 0);
    if (md->sock < 0) {
        fprintf(stderr, "ERROR opening socket");
        return NULL;
    }
    server = gethostbyname(md->server);
    if (server == NULL) {
        fprintf(stderr, "ERROR, no such host\n");
        return NULL;
    }
    bzero((char *)&serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
    serv_addr.sin_port = htons(md->port);

    if (connect(md->sock, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
        printf("Cannot connect to address %s, port %d\n", md->server, md->port);
        printf("Returning from read threads loop\n");
        writelog(md, "read threads stopped.");
        md->sock = -1;
        quit_app_cb(md);
       
    }
    md->lost = 0;

    // get hwserver start time
    if (get_hw_start_time(md, &hw_start_time))
    {
        pthread_mutex_lock(md->hw_start_time_mutex);
        md->hw_start_time = hw_start_time;
        pthread_mutex_unlock(md->hw_start_time_mutex);
    }

    while (1) {
        if (md->closingapp) {
            close(md->sock);
            printf("socket port closed\n");
            return NULL;
        }
       
        if (md->lost) {
            printf("lost connection!\n");
            return NULL;
        }
 
        if (md->mode != mode) { //switch mode if needed
           set_mode(md, md->mode); 
           mode = md->mode;
        }

        
        if (md->running && get_values(md->sock, md->sock_mutex, &xpos, &ypos, &zpiezo, &errorsignal, &in1, &in2, &timestamp, &a1, &p1, in, &md->lost))
        {
           pthread_mutex_lock(md->cap_mutex);
           md->xpos_actual = xpos;
           md->ypos_actual = ypos;
           pthread_mutex_unlock(md->cap_mutex);

           pthread_mutex_lock(md->zpiezo_mutex);
           md->zpiezo_actual = zpiezo;
           md->error_actual = errorsignal;
           md->in1_actual = in1;
           md->in2_actual = in2;
           md->time_actual = timestamp;
           md->a1_actual = a1;
           md->p1_actual = p1;
           for (i=0; i<16; i++) md->in_actual[i] = in[i];
    //       printf("act %g %g\n", md->error_actual, md->in1_actual);
           pthread_mutex_unlock(md->zpiezo_mutex);
        } else {
           pthread_mutex_lock(md->cap_mutex);
           md->xpos_actual = 0;
           md->ypos_actual = 0;
           pthread_mutex_unlock(md->cap_mutex);

           pthread_mutex_lock(md->zpiezo_mutex);
           md->zpiezo_actual = 0;
           md->error_actual = 0;
           md->in1_actual = 0;
           md->in2_actual = 0;
           md->time_actual = 0;
           md->a1_actual = 0;
           md->p1_actual = 0;
           for (i=0; i<16; i++) md->in_actual[i] = 0;
           pthread_mutex_unlock(md->zpiezo_mutex);

        }

        usleep(1000);
   
    }

    close(md->sock);
    printf("socket port closed\n");

   return NULL;
}

gboolean get_array(GwyfileObject *response, gchar *key, gdouble *vdata, gint elength, gint shift)
{
    gint i, length;
    GwyfileItem *valueitem;
    const gdouble *vals;

    valueitem = gwyfile_object_get(response, key);
    if (!valueitem) fprintf(stderr, "Error! message does not contain %s\n", key);
    else {
         vals = gwyfile_item_get_double_array(valueitem);
         length = gwyfile_item_array_length(valueitem);
         if (length!=elength) fprintf(stderr, "Error! number of \"%s\" points returned is not the expected value: %d instead of %d\n", key, length, elength); 
         for (i=0; i<length; i++) vdata[shift + i] = vals[i];

    //     if (strcmp(key, "y")==0) for (i=0; i<length; i++) printf("vals %d %d %g\n", i, shift+i, vals[i]);
    }
    return TRUE;
}

/*get data points for adaptive scan, take range from-to and place it to position in local arrays starting at pos*/
//vdata is the flat field to be filled, ndata is the update number of points filled, npos is allocated number of positions for each channel,
//from and to is the range of positions to request from the server and pos is the place where to put them.
//If lift is non-zero, the standard fields are skipped and only lift channels are filled.
//If xypos data are provided, functions fills the xydata path array with measured x and y positions. This is used in case when the xydata are not known in advance
gint update_scan_path_data(MiniData *md, gdouble *vdata, gint *ndata, gint npos, gint from, gint to, gint pos, gint lift, gdouble *xypos) 
{
    gint rfrom, rto, n, i, k;
    const char *todo;
    char key[20];
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;

    rfrom = rto = 0;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get_scan_data"),
                           gwyfile_item_new_int32("from", from),
                           gwyfile_item_new_int32("to", to),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "get_scan_data") == 0) {

       valueitem = gwyfile_object_get(response, "from");
       if (!valueitem) fprintf(stderr, "Error! get_scan_data message does not contain from\n");
       else {
           rfrom = gwyfile_item_get_int32(valueitem);
       }
       valueitem = gwyfile_object_get(response, "to");
       if (!valueitem) fprintf(stderr, "Error! get_scan_data message does not contain to\n");
       else {
           rto = gwyfile_item_get_int32(valueitem);
       }
       if (rto!=to || rfrom!=from) fprintf(stderr, "Error! get_scan_data range does not match request %d to %d, received %d to %d\n", from, to, rfrom, rto);

       n = 0;
       if (md->c_pr_x) {
           if (lift==0) get_array(response, "x", vdata, to-from, n*npos + pos);
           if (xypos) {
               for (i=0; i<(to-from); i++) xypos[2*(pos+i)] = vdata[n*npos + pos + i];
           }
           n++;
       }
       if (md->c_pr_y) {
           if (lift==0) get_array(response, "y", vdata, to-from, n*npos + pos);
           if (xypos) {
               for (i=0; i<(to-from); i++) xypos[2*(pos+i) + 1] = vdata[n*npos + pos + i];
           }
           n++;
       }
       if (md->c_pr_z) {
           if (lift==0) get_array(response, "z", vdata, to-from, n*npos + pos);
           n++;
       }
       if (md->c_pr_e) {
           if (lift==0) get_array(response, "e", vdata, to-from, n*npos + pos);
           n++;
       }
       if (md->c_pr_ts) {
           if (lift==0) get_array(response, "ts", vdata, to-from, n*npos + pos);
           n++;
       }
       if (md->c_sc_a1) {
           if (lift==0) get_array(response, "a1", vdata, to-from, n*npos + pos);   
           n++;
       }
       if (md->c_sc_p1) {
           if (lift==0) get_array(response, "p1", vdata, to-from, n*npos + pos);
           n++;
       }
       if (md->c_sc_a2) {
           if (lift==0) get_array(response, "a2", vdata, to-from, n*npos + pos);
           n++;
       }
       if (md->c_sc_p2) {
           if (lift==0) get_array(response, "p2", vdata, to-from, n*npos + pos);
           n++;
       }
       for (i=0; i<16; i++) {
          if (md->c_sc_in[i]) {
             if (lift==0) {
                snprintf(key, sizeof(key), "in%d", i+1);
                get_array(response, key, vdata, to-from, n*npos + pos);
             }
             n++;
          }
       }

       for (k=1; k<=lift; k++)
       {
          if (md->c_sc_a1) {
              if (k==lift) get_array(response, "a1", vdata, to-from, n*npos + pos);
              n++;
          }
          if (md->c_sc_p1) {
              if (k==lift) get_array(response, "p1", vdata, to-from, n*npos + pos);
              n++;
          }
          if (md->c_sc_a2) {
              if (k==lift) get_array(response, "a2", vdata, to-from, n*npos + pos);
              n++;
          }
          if (md->c_sc_p2) {
              if (k==lift) get_array(response, "p2", vdata, to-from, n*npos + pos);
              n++;
          }
          for (i=0; i<16; i++) {
             if (md->c_sc_in[i]) {
                if (k==lift) {
                   snprintf(key, sizeof(key), "in%d", i+1);
                   get_array(response, key, vdata, to-from, n*npos + pos);
                }
                n++;
             }
          }
       }


       if (lift==0) *ndata += to-from;   //this must be wrong if we are at lift!
       
    }
    gwyfile_object_free(response);

    return 0;
}

gboolean stop_scan(MiniData *md)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "stop_scan"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

gboolean pause_scan(MiniData *md, gboolean pause)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "pause_scan"),
                           gwyfile_item_new_bool("pause", pause),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}


/*issue command to scan whole image using adaptive path, already created and residing at hw->xypos */
gboolean run_scan_adaptive(MiniData *md)
{
    GwyfileObject *gwyf, *response;
    double *sendpos;
    int i, n, from, to;
    int npos = md->npos;
    int npossend = ceil((int)npos/1000.0);

    sendpos = (double *)malloc(2*npossend*sizeof(double));
    n = 0;
    do {
       i = 0;
       from = n;
       do {
           sendpos[i++] = md->xypos[2*n];
           sendpos[i++] = md->xypos[2*n+1];
           n++;
       } while ((n<npos) && i<(2*npossend));
       to = n;

       gwyf = gwyfile_object_new("GS",
                       gwyfile_item_new_string_copy("todo", "set_scan_path_data"),
                       gwyfile_item_new_int32("n", npos),
                       gwyfile_item_new_int32("from", from),
                       gwyfile_item_new_int32("to", to),
                       gwyfile_item_new_double_array_copy("xydata", sendpos, 2*npossend),
                       NULL);

      if (gwyf==NULL) printf("Error, null\n");

       pthread_mutex_lock(md->sock_mutex);
       response = send_gwy_message(md->sock, gwyf);
       pthread_mutex_unlock(md->sock_mutex);

       gwyfile_object_free(gwyf);
       if (response==NULL) {
          md->sock = -1;
          md->lost = 1;
          return FALSE;
       }
       gwyfile_object_free(response);

    } while (n<npos);


    printf("data passed, run scan adaptive will be issued on socket %d\n", md->sock);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "run_scan_path"),
                           gwyfile_item_new_int32("n", md->npos),
                           NULL);

    if (gwyf==NULL) printf("Error, null\n");

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
    gwyfile_object_free(response);
    free(sendpos);

    return TRUE;
}

/*issue command to scan whole image using lua script */
gboolean run_scan_script(MiniData *md, gchar *script)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "run_scan_script"),
                           gwyfile_item_new_int32("n", md->npos),
                           gwyfile_item_new_string_copy("script", script),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

/*add or set a scanning script parameter*/
gboolean set_scanning_script_parameter(MiniData *md, gchar *name, gdouble value)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "set_script_param"),
                           gwyfile_item_new_string_copy("key", name),
                           gwyfile_item_new_double("value", value),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;

}

/*clear scanning script parameters*/
gboolean clear_scanning_script_parameters(MiniData *md)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "clear_script_params"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}



/*issue command to scan single image line, defined by end points*/
gboolean run_scan_line(MiniData *md, gdouble xfrom, gdouble yfrom, gdouble xto, gdouble yto, gint n, gdouble *zdata)
{
    GwyfileObject *gwyf, *response;
    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "run_scan_line"),
                           gwyfile_item_new_int32("n", n),
                           gwyfile_item_new_double("xfrom", xfrom),
                           gwyfile_item_new_double("yfrom", yfrom),
                           gwyfile_item_new_double("xto", xto),
                           gwyfile_item_new_double("yto", yto),
                           NULL);

    if (zdata) gwyfile_object_add(gwyf, gwyfile_item_new_double_array_copy("z", zdata, n));

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
     gwyfile_object_free(response);

    return TRUE;
}

const gchar* get_version(MiniData *md)
{    
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;
    const gchar *todo;
    const gchar *version;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get"),
                           gwyfile_item_new_string_copy("version", "whatever"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        return "unknown";
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "get") == 0) {
           valueitem = gwyfile_object_get(response, "version");
           if (!valueitem) {
               fprintf(stderr, "Error! read message does not contain version\n");
               return "unknown";
           }
           else {
               version = g_strdup(gwyfile_item_get_string(valueitem));
           }
    }
    else version = g_strdup("unknown");

    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return version;
}

gint get_ramp_ndata(MiniData *md)
{    
    gint n = -1;
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;
    const char *todo;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get_ramp_ndata"),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "get_ramp_ndata") == 0) {
           valueitem = gwyfile_object_get(response, "n");
           if (!valueitem) fprintf(stderr, "Error! read message does not contain n\n");
           else {
               n = gwyfile_item_get_int32(valueitem);
           }
    }
    gwyfile_object_free(gwyf);
    gwyfile_object_free(response);

    return n;
}

/*get data points for ramp, take range from-to and place it to position in local arrays starting at pos*/
//vdata is the flat field to be filled, ndata is the update number of points filled, npos is allocated number of positions for each channel,
//from and to is the range of positions to request from the server and pos is the place where to put them.
gint update_ramp_data(MiniData *md, gdouble *vdata, gint *ndata, gint npos, gint from, gint to, gint pos, gint quantity)
{
    gint rfrom, rto, n, i;
    const char *todo;
    char key[20];
    GwyfileObject *gwyf, *response;
    GwyfileItem *item, *valueitem;

    rfrom = rto = 0;

 //   printf("update ramp from %d to %d   pos %d\n", from, to, pos);

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "get_ramp_data"),
                           gwyfile_item_new_int32("from", from),
                           gwyfile_item_new_int32("to", to),
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf);
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
 
    item = gwyfile_object_get(response, "todo");
    if (!item) {
        fprintf(stderr, "Wrong command, it has no todo.\n");
        response = gwyfile_object_new("GS",
                   gwyfile_item_new_string_copy("todo", "failed, no todo"),
                   NULL);
        return TRUE;
    }

    todo = gwyfile_item_get_string(item);
    if (strcmp(todo, "get_ramp_data") == 0) {

       valueitem = gwyfile_object_get(response, "from");
       if (!valueitem) fprintf(stderr, "Error! get_ramp_data message does not contain from\n");
       else {
           rfrom = gwyfile_item_get_int32(valueitem);
       }
       valueitem = gwyfile_object_get(response, "to");
       if (!valueitem) fprintf(stderr, "Error! get_ramp_data message does not contain to\n");
       else {
           rto = gwyfile_item_get_int32(valueitem);
       }
       if (rto!=to || rfrom!=from) fprintf(stderr, "Error! get_ramp_data range does not match request %d to %d, received %d to %d\n", from, to, rfrom, rto);

       n = 0;
       //add the ramping quantity as zeroth channel
       get_array(response, "q", vdata, to-from, (n++)*npos + pos);
       if (md->c_pr_x) get_array(response, "x", vdata, to-from, (n++)*npos + pos);
       if (md->c_pr_y) get_array(response, "y", vdata, to-from, (n++)*npos + pos);
       if (md->c_pr_z) get_array(response, "z", vdata, to-from, (n++)*npos + pos);
       if (md->c_pr_e) get_array(response, "e", vdata, to-from, (n++)*npos + pos);
       if (md->c_pr_ts) get_array(response, "ts", vdata, to-from, (n++)*npos + pos);
       if (md->c_sc_a1) get_array(response, "a1", vdata, to-from, (n++)*npos + pos);   
       if (md->c_sc_p1) get_array(response, "p1", vdata, to-from, (n++)*npos + pos);
       if (md->c_sc_a2) get_array(response, "a2", vdata, to-from, (n++)*npos + pos);
       if (md->c_sc_p2) get_array(response, "p2", vdata, to-from, (n++)*npos + pos);
       for (i=0; i<16; i++) {
          if (md->c_sc_in[i]) {
              snprintf(key, sizeof(key), "in%d", i+1);
              get_array(response, key, vdata, to-from, (n++)*npos + pos);
          }
       }

       *ndata += to-from;
    }

    return 0;
}

static
const char* ramp_quantity_to_string(int rule)
{
    if (rule==GWS_RAMP_ZPIEZO) return "z";
    else if (rule==GWS_RAMP_TIPV) return "out1";
    else if (rule==GWS_RAMP_SAMPLEV) return "out2";
    else if (rule==GWS_RAMP_OUT1) return "out1";
    else if (rule==GWS_RAMP_OUT2) return "out2";
    else return "nothing";
}

/*issue command to run a single ramp*/
gboolean run_ramp(MiniData *md, gint quantity, gint n, gdouble from, gdouble to, gdouble start_delay, gdouble peak_delay, gdouble time_up, gdouble time_down)
{
    GwyfileObject *gwyf, *response;
    gdouble scale = 1;

    if (quantity==GWS_RAMP_ZPIEZO) scale = 1e-6;

    gwyf = gwyfile_object_new("GS",
                           gwyfile_item_new_string_copy("todo", "run_ramp"),
                           gwyfile_item_new_int32("n", n),
                           gwyfile_item_new_string_copy("quantity", ramp_quantity_to_string(quantity)),
                           gwyfile_item_new_double("from", scale*from),
                           gwyfile_item_new_double("to", scale*to),
                           gwyfile_item_new_double("start_delay", start_delay*1e-3), //it was in ms
                           gwyfile_item_new_double("peak_delay", peak_delay*1e-3), //it was in ms
                           gwyfile_item_new_double("time_up", time_up*1e-3),    //it was in ms
                           gwyfile_item_new_double("time_down", time_down*1e-3), //it was in ms
                           NULL);

    pthread_mutex_lock(md->sock_mutex);
    response = send_gwy_message(md->sock, gwyf); 
    pthread_mutex_unlock(md->sock_mutex);

    gwyfile_object_free(gwyf);
    if (response==NULL) {
       md->sock = -1;
       md->lost = 1;
       return FALSE;
    }
    gwyfile_object_free(response);

    return TRUE;
}


void
start_read_threads(MiniData *md)
{
    md->hw_loop = g_thread_new("gwyminiscan_hwthread", hw_loop, md);    
}


