
#ifndef MINISCAN_HW
#define MINISCAN_HW

#include "data.h"


void start_read_threads(MiniData *md);

gboolean set_standby_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd);
gboolean set_scan_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd);
gboolean set_ramp_storage(MiniData *md, gboolean a2, gboolean p2, gboolean in[16], gboolean fmd);

gint get_scan_ndata(MiniData *md);
gboolean run_scan_adaptive(MiniData *md);
gboolean run_scan_line(MiniData *md, gdouble xfrom, gdouble yfrom, gdouble xto, gdouble yto, gint n, gdouble *zdata);
gboolean run_scan_script(MiniData *md, gchar *script);

gint update_scan_path_data(MiniData *md, gdouble *vdata, gint *ndata, gint npos, gint from, gint to, gint pos, gint lift, gdouble *xypos);
gboolean stop_scan(MiniData *md);
gboolean pause_scan(MiniData *md, gboolean pause);

const gchar* get_version(MiniData *md);

gint get_ramp_ndata(MiniData *md);
gboolean run_ramp(MiniData *md, gint quantity, gint n, gdouble from, gdouble to, gdouble start_delay, gdouble peak_delay, gdouble time_up, gdouble time_down);
gint update_ramp_data(MiniData *md, gdouble *vdata, gint *ndata, gint npos, gint from, gint to, gint pos, gint quantity);

/*commands to control the hardware*/
gboolean table_move_to(MiniData *md, gdouble xpos, gdouble ypos);
gboolean set_analog_value(MiniData *md, gint channel, gdouble val);
gboolean set_speed(MiniData *md, gdouble val);
gboolean set_wait(MiniData *md, gdouble val);
gboolean set_setpoint(MiniData *md, gdouble setpoint);
gboolean set_pid(MiniData *md, gdouble pidp, gdouble pidi, gdouble pidd);
gboolean set_fmpid(MiniData *md, gdouble pidp, gdouble pidi, gdouble pidd);
gboolean set_fmsetpoint(MiniData *md, gdouble setpoint);
gboolean set_freq(MiniData *md, gint channel, gdouble frequency, gdouble amplitude, gdouble phase);
gboolean set_routing(MiniData *md, const gchar *channel, gint rule);
gboolean slide_freq(MiniData *md, gint channel, gdouble frequency);

gboolean set_filter(MiniData *md, gint channel, gint val);

gboolean set_feedback(MiniData *md, gboolean feedback);
gboolean set_fmfeedback(MiniData *md, gboolean feedback);
gboolean set_zpiezo(MiniData *md, gdouble zpiezo);
gboolean set_fmlimit(MiniData *md, gint fmlimit);
gboolean set_phaselimit(MiniData *md, gint phaselimit);
gboolean set_filterphase(MiniData *md, gboolean filterphase);
gboolean set_fmam_feedback(MiniData *md, gboolean feedback);
gboolean set_phaseshift(MiniData *md, gint phaseshift);

gboolean set_mode(MiniData *md, gint mode);

gboolean set_motor_up(MiniData *md);
gboolean set_motor_down(MiniData *md);

gboolean reset_start_time(MiniData *md);
gboolean get_hw_start_time(MiniData *md, gdouble *hw_start_time);
gboolean get_scanning_script_status(MiniData *md);
gboolean set_scanning_script_parameter(MiniData *md, gchar *name, gdouble value);
gboolean clear_scanning_script_parameters(MiniData *md);

gboolean is_table_to(MiniData *md, gboolean *is_table_to);

#endif

