/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <app/gwymoduleutils.h>
#include <app/gwyapp.h>
#include "laserview.h"
#include "data.h"
#include "hw.h"

static void gws_laserview_class_init          (GwsLaserViewClass *klass);
static void gws_laserview_init                (GwsLaserView      *ttt);

GType
gws_laserview_get_type (void)
{
  static GType laserview_type = 0;

  if (!laserview_type)
    {
      const GTypeInfo laserview_info =
      {
	sizeof (GwsLaserViewClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) gws_laserview_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (GwsLaserView),
	0,
	(GInstanceInitFunc) gws_laserview_init,
      };

      laserview_type = g_type_register_static (GTK_TYPE_TABLE, "GwsLaserView", &laserview_info, 0);
    }

  return laserview_type;
}

static void
gws_laserview_class_init (GwsLaserViewClass *klass)
{
}


static gboolean
laserupdate(gpointer data)
{
    gchar buffer[400];
    gdouble xval, yval, buf;
    GwsLaserView *fv = (GwsLaserView *)data;
    MiniData *md = (MiniData *)fv->md;
    gdouble xpos, ypos;

    if (md->closingapp || fv->closing)
        return FALSE;

    get_actual_zdata(md, &buf, &buf, &xval, &yval, &buf, &buf, &buf);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", xval);
    gtk_label_set_text(GTK_LABEL(fv->xlabel), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", yval);
    gtk_label_set_text(GTK_LABEL(fv->ylabel), buffer);

    
    gwy_data_field_fill(fv->dfield, 0);
   
    xpos = 100*xval/20 + 100; 
    ypos = 100*yval/20 + 100; 

    if (xpos<5) xpos = 5;
    if (ypos<5) ypos = 5;
    if (xpos>194) xpos = 194;
    if (ypos>194) ypos = 194;
 
    gwy_data_field_area_fill(fv->dfield, xpos-5, ypos-5, 10, 10, 10.0);
    gwy_data_field_data_changed(fv->dfield);


    return TRUE;
}

static void
gws_laserview_init (GwsLaserView *laserview)
{
    GtkWidget *label;
    GwyPixmapLayer *layer;
    gchar key[32];
    guint row = 0;


    gtk_table_resize (GTK_TABLE(laserview), 6, 4);

    label = gtk_label_new("Image:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(laserview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    row++;

    g_snprintf(key, sizeof(key), "/0/data");

    laserview->container = gwy_container_new();
    laserview->dfield = gwy_data_field_new(200, 200, 200, 200, 1);

    g_snprintf(key, sizeof(key), "/0/data");
    gwy_container_set_object_by_name(laserview->container, key, laserview->dfield);

    laserview->dataview = GWY_DATA_VIEW(gwy_data_view_new(laserview->container));
    gwy_data_view_set_data_prefix(GWY_DATA_VIEW(laserview->dataview), key);
    layer = gwy_layer_basic_new();
    g_object_set(layer,
                 "data-key", "/0/data",
                 "gradient-key", "/0/base/palette",
                 NULL);

    gwy_data_view_set_base_layer(GWY_DATA_VIEW(laserview->dataview), layer);
    gwy_pixmap_layer_set_data_key(layer, key);

    gwy_set_data_preview_size(GWY_DATA_VIEW(laserview->dataview), 200);
    gtk_table_attach(GTK_TABLE(laserview), GTK_WIDGET(laserview->dataview),
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("X:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(laserview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


    laserview->xlabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(laserview->xlabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(laserview), laserview->xlabel,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("Y:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(laserview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    laserview->ylabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(laserview->ylabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(laserview), laserview->ylabel,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;


  gtk_widget_show_all(GTK_WIDGET(laserview));

  g_timeout_add(100, laserupdate, laserview);


}

GtkWidget*
gws_laserview_new(gpointer md)
{
  GwsLaserView *fv = g_object_new (gws_laserview_get_type (), NULL);
  fv->md = md;
  fv->closing = 0;

  return GTK_WIDGET(fv);
}


