/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c
 * Copyright (C) Petr Klapetek 2007-2020 <pklapetek@cmi.cz>
 * 
 * main.c is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * main.c is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with main.c.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
//#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <X11/Xlib.h>

//#include <config.h>

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <app/gwyapp.h>

#ifdef G_OS_WIN32
#include <windows.h>
#include <winreg.h>
#else
#endif

#include "data.h"
#include "gui.h"

void load_keyfile_data(MiniData *md, GKeyFile *keyfile)
{
    md->server = g_key_file_get_string(keyfile, "Servers",
                                                   "hardware_server", NULL);
    md->port = g_key_file_get_integer(keyfile, "Servers",
                                                   "port", NULL);

    md->wait = g_key_file_get_integer(keyfile, "General", "wait", NULL);
    md->speed = g_key_file_get_double(keyfile, "General", "speed", NULL);
    md->preview_step = g_key_file_get_integer(keyfile, "General", "preview_step", NULL);
    md->filter1 = g_key_file_get_integer(keyfile, "General", "filter1", NULL);
    md->filter2 = g_key_file_get_integer(keyfile, "General", "filter2", NULL);

    md->rxres = g_key_file_get_integer(keyfile, "Regular", "xres", NULL);
    md->ryres = g_key_file_get_integer(keyfile, "Regular", "yres", NULL);
    md->rxreal = g_key_file_get_double(keyfile, "Regular", "xreal", NULL);
    md->ryreal = g_key_file_get_double(keyfile, "Regular", "yreal", NULL);
    md->rxoff = g_key_file_get_double(keyfile, "Regular", "xoff", NULL);
    md->ryoff = g_key_file_get_double(keyfile, "Regular", "yoff", NULL);
    md->rrotate = g_key_file_get_double(keyfile, "Regular", "rotate", NULL);

    printf("xoff %g yoff %g", md->rxoff, md->ryoff);

    md->xres = g_key_file_get_integer(keyfile, "Adaptive", "xres", NULL);
    md->yres = g_key_file_get_integer(keyfile, "Adaptive", "yres", NULL);
    md->xreal = g_key_file_get_double(keyfile, "Adaptive", "xreal", NULL);
    md->yreal = g_key_file_get_double(keyfile, "Adaptive", "yreal", NULL);
    md->xoff = g_key_file_get_double(keyfile, "Adaptive", "xoff", NULL);
    md->yoff = g_key_file_get_double(keyfile, "Adaptive", "yoff", NULL);
    md->rotate = g_key_file_get_double(keyfile, "Adaptive", "rotate", NULL);
    md->nvals = g_key_file_get_integer(keyfile, "Adaptive", "nvals", NULL);
    md->order = g_key_file_get_integer(keyfile, "Adaptive", "order", NULL);
    md->nprofiles = g_key_file_get_integer(keyfile, "Adaptive", "nprofiles", NULL);
    md->noctaves = g_key_file_get_integer(keyfile, "Adaptive", "noctaves", NULL);

    md->freq_from = g_key_file_get_double(keyfile, "Frequency", "freq_from", NULL);
    md->freq_to = g_key_file_get_double(keyfile, "Frequency", "freq_to", NULL);
    md->freq_step = g_key_file_get_double(keyfile, "Frequency", "freq_step", NULL);
    md->freq_delay = g_key_file_get_double(keyfile, "Frequency", "freq_delay", NULL);
    md->freq_aout0 = g_key_file_get_double(keyfile, "Frequency", "freq_aout0", NULL);
    md->exc1 = g_key_file_get_double(keyfile, "Frequency", "exc1", NULL);
    md->exc2 = g_key_file_get_double(keyfile, "Frequency", "exc2", NULL);
    md->phaseshift1 = g_key_file_get_integer(keyfile, "Frequency", "phaseshift1", NULL);
    md->phaseshift2 = g_key_file_get_integer(keyfile, "Frequency", "phaseshift2", NULL);

    md->pidp = g_key_file_get_double(keyfile, "PID", "pidp", NULL);
    md->pidi = g_key_file_get_double(keyfile, "PID", "pidi", NULL);
    md->pidd = g_key_file_get_double(keyfile, "PID", "pidd", NULL);
    md->fmpidp = g_key_file_get_double(keyfile, "PID", "fmpidp", NULL);
    md->fmpidi = g_key_file_get_double(keyfile, "PID", "fmpidi", NULL);
    md->fmpidd = g_key_file_get_double(keyfile, "PID", "fmpidd", NULL);
    md->fm_frequency_limit = g_key_file_get_integer(keyfile, "PID", "fm_freq_factor", NULL);
    md->fm_phase_limit = g_key_file_get_integer(keyfile, "PID", "fm_phase_factor", NULL);
    md->phasefilter = g_key_file_get_integer(keyfile, "PID", "fm_phase_filter", NULL);

}

void    
save_keyfile_data(MiniData *md, GKeyFile *keyfile)
{

    g_key_file_set_string(keyfile, "Servers", "hardware_server", md->server);
    g_key_file_set_integer(keyfile, "Servers", "port", md->port);

    g_key_file_set_integer(keyfile, "General", "wait", md->wait);
    g_key_file_set_double(keyfile, "General", "speed", md->speed);
    g_key_file_set_integer(keyfile, "General", "preview_step", md->preview_step);
    g_key_file_set_integer(keyfile, "General", "filter1", md->filter1);
    g_key_file_set_integer(keyfile, "General", "filter2", md->filter2);

    g_key_file_set_integer(keyfile, "Regular", "xres", md->rxres);
    g_key_file_set_integer(keyfile, "Regular", "yres", md->ryres);
    g_key_file_set_double(keyfile, "Regular", "xreal", md->rxreal);
    g_key_file_set_double(keyfile, "Regular", "yreal", md->ryreal);
    g_key_file_set_double(keyfile, "Regular", "xoff", md->rxoff);
    g_key_file_set_double(keyfile, "Regular", "yoff", md->ryoff);
    g_key_file_set_double(keyfile, "Regular", "rotate", md->rrotate);

    g_key_file_set_integer(keyfile, "Adaptive", "xres", md->xres);
    g_key_file_set_integer(keyfile, "Adaptive", "yres", md->yres);
    g_key_file_set_double(keyfile, "Adaptive", "xreal", md->xreal);
    g_key_file_set_double(keyfile, "Adaptive", "yreal", md->yreal);
    g_key_file_set_double(keyfile, "Adaptive", "xoff", md->xoff);
    g_key_file_set_double(keyfile, "Adaptive", "yoff", md->yoff);
    g_key_file_set_double(keyfile, "Adaptive", "rotate", md->rotate);
    g_key_file_set_integer(keyfile, "Adaptive", "nvals", md->nvals);
    g_key_file_set_integer(keyfile, "Adaptive", "order", md->order);
    g_key_file_set_integer(keyfile, "Adaptive", "nprofiles", md->nprofiles);
    g_key_file_set_integer(keyfile, "Adaptive", "noctaves", md->noctaves);

    g_key_file_set_double(keyfile, "Frequency", "freq_from", md->freq_from);
    g_key_file_set_double(keyfile, "Frequency", "freq_to", md->freq_to);
    g_key_file_set_double(keyfile, "Frequency", "freq_step", md->freq_step);
    g_key_file_set_double(keyfile, "Frequency", "freq_delay", md->freq_delay);
    g_key_file_set_double(keyfile, "Frequency", "freq_aout0", md->freq_aout0);
    g_key_file_set_double(keyfile, "Frequency", "exc1", md->exc1);
    g_key_file_set_double(keyfile, "Frequency", "exc2", md->exc2);
    g_key_file_set_integer(keyfile, "Frequency", "phaseshift1", md->phaseshift1);
    g_key_file_set_integer(keyfile, "Frequency", "phaseshift2", md->phaseshift2);

    g_key_file_set_double(keyfile, "PID", "pidp", md->pidp);
    g_key_file_set_double(keyfile, "PID", "pidi", md->pidi);
    g_key_file_set_double(keyfile, "PID", "pidd", md->pidd);
    g_key_file_set_double(keyfile, "PID", "fmpidp", md->fmpidp);
    g_key_file_set_double(keyfile, "PID", "fmpidi", md->fmpidi);
    g_key_file_set_double(keyfile, "PID", "fmpidd", md->fmpidd);
    g_key_file_set_integer(keyfile, "PID", "fm_freq_factor", md->fm_frequency_limit);
    g_key_file_set_integer(keyfile, "PID", "fm_phase_factor", md->fm_phase_limit);
    g_key_file_set_integer(keyfile, "PID", "fm_phase_filter", md->phasefilter);

}



int
main (int argc, char *argv[])
{

    MiniData md;
    GKeyFile *keyfile;
    GKeyFileFlags flags;
    gchar *config_path;
    GError *error = NULL;
    gsize size;
    gchar *data;

    gdk_threads_init();
    gdk_threads_enter(); 
    XInitThreads(); /* see http://stackoverflow.com/a/18690540/472927 */


    gtk_set_locale();
    gtk_init (&argc, &argv);

    gwy_app_init_common(NULL, "layer", "file", NULL);

    if (init_data(&md, MINISCAN_INI_FOLDER, PREVIEW_SIZE_PIXELS, NULL) == FALSE)  //thread function should be the last argument, no idea what it does
        return 0;
 

    config_path = g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, MINISCAN_INI_FILE_NAME, NULL);
    printf("settings path: (%s)", config_path);
    keyfile = g_key_file_new();
    flags = G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS;

    if (!g_key_file_load_from_file(keyfile, config_path, flags, &error))
    {  
       printf("No configuration file found (%s)\n", error->message);
       g_error_free(error);

       md.server = g_strdup("192.168.9.16");
       md.port = 123456;

       printf("Trying to create configuration file.\n");
       if (!g_file_test(g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, NULL), G_FILE_TEST_EXISTS))
           g_mkdir(g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, NULL), 0777);


       g_key_file_set_string(keyfile, "Servers", "hardware_server", md.server);
       g_key_file_set_integer(keyfile, "Servers", "port", md.port);

       data = g_key_file_to_data(keyfile, &size, &error);
       if (!data) {
           printf("Cannot create configuration file: %s\n", error->message);
           g_error_free(error);
       }
       else {
          error = NULL;
          if (!g_file_set_contents (config_path, data, size, &error))
          {
  	       printf("Cannot save configuration file to %s: %s\n", config_path, error->message);
	       g_error_free(error);

          } else {
  	       printf("Configuration file saved.\n");
          }
       }


    } else {
       load_keyfile_data(&md, keyfile);

       printf("Sever/port loaded from configuration file: %s:%d\n", md.server, md.port);

    }

    start_logging_and_threads(&md);

    //if (init_data(&md, MINISCAN_INI_FOLDER, PREVIEW_SIZE_PIXELS, NULL) == FALSE)  //thread function should be the last argument, no idea what it does
    //    return 0;
        
    create_window(&md);
    init_hw(&md);
    gtk_main();
    gdk_threads_leave();

    //save settings
    printf("saving settings to (%s)\n", config_path);
    save_keyfile_data(&md, keyfile);

    data = g_key_file_to_data(keyfile, &size, &error);
    if (!data) {
        printf("Cannot create configuration file: %s\n", error->message);
        g_error_free(error);
    }
    else {
       error = NULL;
       if (!g_file_set_contents (config_path, data, size, &error))
       {
            printf("Cannot save configuration file to %s: %s\n", config_path, error->message);
            g_error_free(error);

       } else {
           printf("Configuration file saved.\n");
       }
    }


    free_data(&md);

    return 0;
}

/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
