/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <app/gwymoduleutils.h>
#include <app/gwyapp.h>
#include "signals.h"
#include "data.h"
#include "hw.h"

typedef enum {
    FILTER_5  = 5,
    FILTER_6  = 6,
    FILTER_7  = 7,
    FILTER_8  = 8,
    FILTER_9  = 9,
    FILTER_10 = 10,
    FILTER_11 = 11,
    FILTER_12 = 12
} GwsFilter;

static void gws_signals_class_init          (GwsSignalsClass *klass);
static void gws_signals_init                (GwsSignals      *ttt);

GType
gws_signals_get_type (void)
{
  static GType signals_type = 0;

  if (!signals_type)
    {
      const GTypeInfo signals_info =
      {
	sizeof (GwsSignalsClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) gws_signals_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (GwsSignals),
	0,
	(GInstanceInitFunc) gws_signals_init,
      };

      signals_type = g_type_register_static (GTK_TYPE_TABLE, "GwsSignals", &signals_info, 0);
    }

  return signals_type;
}

static void
gws_signals_class_init (GwsSignalsClass *klass)
{
}


static gboolean
laserupdate(gpointer data)
{
    gchar buffer[400];
    gdouble xval, yval, error, a1, p1, zpiezo, in1, in2, buf;
    gdouble ins[16];
    GwsSignals *fv = (GwsSignals *)data;
    MiniData *md = (MiniData *)fv->md;

    if (md->closingapp || fv->closing)
        return FALSE;

    get_actual_zdata(md, &zpiezo, &error, &in1, &in2, &buf, &a1, &p1);
    get_actual_in(md, ins);
    get_actual_pos(md, &xval, &yval); 

    g_snprintf(buffer, sizeof(buffer), "%3.3f μm", xval*1e6);
    gtk_label_set_text(GTK_LABEL(fv->xlabel), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f μm", yval*1e6);
    gtk_label_set_text(GTK_LABEL(fv->ylabel), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f μm", zpiezo*1e6);
    gtk_label_set_text(GTK_LABEL(fv->zpiezo), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", error);
    gtk_label_set_text(GTK_LABEL(fv->error), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", in1);
    gtk_label_set_text(GTK_LABEL(fv->in1), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", in2);
    gtk_label_set_text(GTK_LABEL(fv->in2), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", a1);
    gtk_label_set_text(GTK_LABEL(fv->a1), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f V", p1);
    gtk_label_set_text(GTK_LABEL(fv->p1), buffer);


    g_snprintf(buffer, sizeof(buffer), "%3.3f %3.3f %3.3f %3.3f V", ins[0], ins[1], ins[2], ins[3]);
    gtk_label_set_text(GTK_LABEL(fv->ins1), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f %3.3f %3.3f %3.3f V", ins[4], ins[5], ins[6], ins[7]);
    gtk_label_set_text(GTK_LABEL(fv->ins2), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f %3.3f %3.3f %3.3f V", ins[8], ins[9], ins[10], ins[11]);
    gtk_label_set_text(GTK_LABEL(fv->ins3), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f %3.3f %3.3f %3.3f V", ins[12], ins[13], ins[14], ins[15]);
    gtk_label_set_text(GTK_LABEL(fv->ins4), buffer);




    return TRUE;
}

static void
filter_changed_cb(GtkComboBox *combo, GwsSignals *signals)
{
    MiniData *md = (MiniData *)signals->md;

    signals->filter = gwy_enum_combo_box_get_active(combo);
    printf("filter set to %d\n", signals->filter);

    set_filter(md, 0, signals->filter);
    md->filter1 = signals->filter;

}

static void
dac_changed_cb(GtkObject *object, GwsSignals *signals)
{
    gint i;
    MiniData *md = (MiniData *)signals->md;

    if (signals->in_update) return;

    for (i=0; i<8; i++) 
    { 
        md->dac[i+8] = gtk_adjustment_get_value(GTK_ADJUSTMENT(signals->dac_adj[i]));
        set_analog_value(md, i+8, md->dac[i+8]);
    }

}

static void
set_inits(GwsSignals *signals)
{
    gint i;
    MiniData *md = (MiniData *)signals->md;

    signals->in_update = TRUE;

    for (i=0; i<8; i++) {
        gtk_adjustment_set_value(GTK_ADJUSTMENT(signals->dac_adj[i]), md->dac[i+8]);
        set_analog_value(md, i+8, md->dac[i+8]);
    }
    signals->in_update = FALSE;
 }


static void
gws_signals_init (GwsSignals *signals)
{
    GtkWidget *label;
    gchar key[32];
    guint row = 0;
    gint i;

    printf("init\n");

   static const GwyEnum filters[] = {
        { N_("5 kHz"),            FILTER_12, },
        { N_("10 kHz"),           FILTER_11, },
        { N_("20 kHz"),           FILTER_10, },
        { N_("35 kHz"),           FILTER_9,  },
        { N_("50 kHz"),           FILTER_8,  },
        { N_("90 kHz"),           FILTER_7,  },
        { N_("175 kHz"),          FILTER_6,  },
        { N_("350 kHz"),          FILTER_5,  },
    };


    gtk_table_resize (GTK_TABLE(signals), 16, 4);

    label = gtk_label_new("XY positioning system:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 4, row, row+1, GTK_FILL, 0, 5, 1);
    row++;


    label = gtk_label_new("X:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


    signals->xlabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->xlabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->xlabel,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("Y:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->ylabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->ylabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->ylabel,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    row++;

    label = gtk_label_new("FPGA data:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 4, row, row+1, GTK_FILL, 0, 5, 1);
    row++;


    signals->filter_combo
        = gwy_enum_combo_box_new(filters, G_N_ELEMENTS(filters),
                                 G_CALLBACK(filter_changed_cb),
                                 signals, signals->filter, TRUE);

    label = gtk_label_new("Low pass filter:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 2, row, row+1, GTK_FILL, 0, 5, 1);

    gtk_table_attach(GTK_TABLE(signals), signals->filter_combo,
                   2, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("IN1:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->in1 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->in1), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->in1,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->in1raw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->in1raw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->in1raw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("IN2:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->in2 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->in2), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->in2,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->in2raw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->in2raw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->in2raw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Error:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->error = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->error), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->error,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->errorraw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->errorraw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->errorraw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Z piezo:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->zpiezo = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->zpiezo), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->zpiezo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->zpiezoraw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->zpiezoraw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->zpiezoraw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Amplitude 1:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->a1 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->a1), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->a1,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->a1raw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->a1raw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->a1raw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Phase 1");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);

    signals->p1 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->p1), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->p1,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("raw:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    signals->p1raw = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->p1raw), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->p1raw,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;


    label = gtk_label_new("Simultaneous A/D data:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 4, row, row+1, GTK_FILL, 0, 5, 1);
    row++;

    signals->ins1 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->ins1), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->ins1,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    signals->ins2 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->ins2), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->ins2,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    signals->ins3 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->ins3), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->ins3,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    signals->ins4 = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(signals->ins4), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), signals->ins4,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;


    label = gtk_label_new("Slow D/A data:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(signals), label,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    signals->in_update = TRUE;


    for (i=0; i<8; i++) {
       snprintf(key, sizeof(key), "Ch %d\n", i+9);
       label = gtk_label_new(key);
       gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
       gtk_table_attach(GTK_TABLE(signals), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

       signals->dac_adj[i] = gtk_adjustment_new(0, -10, 10, 0.01, 0.1, 0);
       signals->dac_spin[i] = gtk_spin_button_new(GTK_ADJUSTMENT(signals->dac_adj[i]), 1, 2);
       gtk_spin_button_set_digits(GTK_SPIN_BUTTON(signals->dac_spin[i]), 3);
       g_signal_connect(signals->dac_adj[i], "value-changed",
                     G_CALLBACK(dac_changed_cb), signals);
       gtk_table_attach(GTK_TABLE(signals), signals->dac_spin[i],
                   1, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

       label = gtk_label_new("V");
       gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
       gtk_table_attach(GTK_TABLE(signals), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

       row++;

   }

   gtk_widget_show_all(GTK_WIDGET(signals));
   g_timeout_add(100, laserupdate, signals);

}

GtkWidget*
gws_signals_new(gpointer md)
{

  GwsSignals *fv = g_object_new (gws_signals_get_type (), NULL);
  fv->md = md;
  fv->filter = ((MiniData *)md)->filter1;
  fv->closing = 0;
  fv->in_update = FALSE;

  set_inits(fv);

  return GTK_WIDGET(fv);
}


