/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>
#include <app/gwymoduleutils.h>
#include <libgwydgets/gwydgetutils.h>
#include <app/gwyapp.h>
#include "tableview.h"
#include "data.h"
#include "hw.h"


static void gws_tableview_class_init          (GwsTableViewClass *klass);
static void gws_tableview_init                (GwsTableView      *ttt);

GType
gws_tableview_get_type (void)
{
  static GType tableview_type = 0;

  if (!tableview_type)
    {
      const GTypeInfo tableview_info =
      {
	sizeof (GwsTableViewClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) gws_tableview_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (GwsTableView),
	0,
	(GInstanceInitFunc) gws_tableview_init,
      };

      tableview_type = g_type_register_static (GTK_TYPE_TABLE, "GwsTableView", &tableview_info, 0);
    }

  return tableview_type;
}

static void
gws_tableview_class_init (GwsTableViewClass *klass)
{
}


static gboolean
tableupdate(gpointer data)
{
    gchar buffer[400];
    gdouble xval, yval;
    GwsTableView *fv = (GwsTableView *)data;
    MiniData *md = (MiniData *)fv->md;
    gdouble xpos, ypos, rxpos, rypos;

    if (md->closingapp || fv->closing)
        return FALSE;

    get_actual_pos(md, &xval, &yval);

    g_snprintf(buffer, sizeof(buffer), "%3.3f μm", xval*1e6);
    gtk_label_set_text(GTK_LABEL(fv->xlabel), buffer);

    g_snprintf(buffer, sizeof(buffer), "%3.3f μm", yval*1e6);
    gtk_label_set_text(GTK_LABEL(fv->ylabel), buffer);

    gwy_data_field_fill(fv->dfield, 0);
   
    xpos = 200*xval/fv->xrange; 
    ypos = 200*yval/fv->yrange; 

    if (xpos<5) xpos = 5;
    if (ypos<5) ypos = 5;
    if (xpos>194) xpos = 194;
    if (ypos>194) ypos = 194;


    rxpos = 200*fv->xreq/fv->xrange; 
    rypos = 200*fv->yreq/fv->yrange; 

    if (rxpos<5) rxpos = 5;
    if (rypos<5) rypos = 5;
    if (rxpos>194) rxpos = 194;
    if (rypos>194) rypos = 194;
  
    gwy_data_field_area_fill(fv->dfield, rxpos-5, rypos-5, 10, 10, 5.0);
    gwy_data_field_area_fill(fv->dfield, xpos-3, ypos-3, 6, 6, 10.0);
    gwy_data_field_data_changed(fv->dfield);

    return TRUE;
}

static void
xpos_changed(GwsTableView *tableview,
                GtkAdjustment *adj)
{
    MiniData *md = (MiniData *)tableview->md;
    tableview->xreq = gtk_adjustment_get_value(adj)*1e-6;
    table_move_to(md, tableview->xreq, tableview->yreq);

}

static void
ypos_changed(GwsTableView *tableview,
                GtkAdjustment *adj)
{
    MiniData *md = (MiniData *)tableview->md;
    tableview->yreq = gtk_adjustment_get_value(adj)*1e-6;
    table_move_to(md, tableview->xreq, tableview->yreq);
}


static void
speed_changed(GwsTableView *tableview,
                GtkAdjustment *adj)
{
    MiniData *md = (MiniData *)tableview->md;
    tableview->speed = gtk_adjustment_get_value(adj);

    md->speed = tableview->speed;
    set_speed(md, tableview->speed);
}

static void
gws_tableview_init (GwsTableView *tableview)
{
    GtkWidget *label;
    gchar key[32];
    GwyPixmapLayer *layer;
    guint row = 0;

    gtk_table_resize (GTK_TABLE(tableview), 6, 4);


    g_snprintf(key, sizeof(key), "/0/data");

    tableview->container = gwy_container_new();
    tableview->dfield = gwy_data_field_new(200, 200, 200, 200, 1);

    g_snprintf(key, sizeof(key), "/0/data");
    gwy_container_set_object_by_name(tableview->container, key, tableview->dfield);

    tableview->dataview = GWY_DATA_VIEW(gwy_data_view_new(tableview->container));
    gwy_data_view_set_data_prefix(GWY_DATA_VIEW(tableview->dataview), key);
    layer = gwy_layer_basic_new();
    g_object_set(layer,
                 "data-key", "/0/data",
                 "gradient-key", "/0/base/palette",
                 NULL);

    gwy_data_view_set_base_layer(GWY_DATA_VIEW(tableview->dataview), layer);
    gwy_pixmap_layer_set_data_key(layer, key);

    gwy_set_data_preview_size(GWY_DATA_VIEW(tableview->dataview), 200);
    gtk_table_attach(GTK_TABLE(tableview), GTK_WIDGET(tableview->dataview),
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("X:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(tableview), label,
                   0, 1, row, row+1, GTK_FILL, 0, 5, 1);


    tableview->xlabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(tableview->xlabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(tableview), tableview->xlabel,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    label = gtk_label_new("Y:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(tableview), label,
                   2, 3, row, row+1, GTK_FILL, 0, 5, 1);

    tableview->ylabel = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(tableview->ylabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(tableview), tableview->ylabel,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    row++;

    tableview->xadj = gtk_adjustment_new(tableview->xreq, 0, tableview->xrange*1e6, 0.1, 10, 0);

    gwy_table_attach_hscale(GTK_WIDGET(tableview), row++, _("_X pos:"), _("μm"),
                            tableview->xadj, GWY_HSCALE_DEFAULT);
    g_signal_connect_swapped(tableview->xadj, "value-changed",
                             G_CALLBACK(xpos_changed), tableview);

    row++;

    tableview->yadj = gtk_adjustment_new(tableview->yreq, 0, tableview->yrange*1e6, 0.1, 10, 0);
    gwy_table_attach_hscale(GTK_WIDGET(tableview), row++, _("_Y pos:"), _("μm"),
                            tableview->yadj, GWY_HSCALE_DEFAULT);
    g_signal_connect_swapped(tableview->yadj, "value-changed",
                             G_CALLBACK(ypos_changed), tableview);

    row++;

    tableview->speedadj = gtk_adjustment_new(tableview->speed, 0.01, 100, 0.1, 10, 0);
    gwy_table_attach_hscale(GTK_WIDGET(tableview), row++, _("_Speed:"), _("μm/s"),
                            tableview->speedadj, GWY_HSCALE_LOG);
    g_signal_connect_swapped(tableview->speedadj, "value-changed",
                             G_CALLBACK(speed_changed), tableview);

    row++;


    gtk_widget_show_all(GTK_WIDGET(tableview));
    g_timeout_add(200, tableupdate, tableview);


}

void
gws_tableview_set_values(GwsTableView *tableview, gdouble xrange, gdouble yrange, gdouble xreq, gdouble yreq, gdouble speed)
{
    tableview->xreq = xreq;
    tableview->yreq = yreq;
    tableview->xrange = xrange;
    tableview->yrange = yrange;
    tableview->speed = speed;

    gtk_adjustment_set_upper(GTK_ADJUSTMENT(tableview->xadj), xrange*1e6);
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(tableview->yadj), yrange*1e6);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(tableview->speedadj), speed);
}

GtkWidget*
gws_tableview_new(gpointer md)
{
  GwsTableView *fv = g_object_new (gws_tableview_get_type (), NULL);
  fv->md = md;
  fv->closing = 0;
  fv->xreq = 0;
  fv->yreq = 0;
  fv->xrange = 1;
  fv->yrange = 1;


  return GTK_WIDGET(fv);
}


