/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __TABLEVIEW_H__
#define __TABLEVIEW_H__


#include <glib.h>
#include <glib-object.h>
#include <gtk/gtktable.h>

#include <libgwyddion/gwyddion.h>
#include <libgwydgets/gwydgets.h>


G_BEGIN_DECLS

#define GWS_TABLEVIEW_TYPE            (gws_tableview_get_type ())
#define GWS_TABLEVIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GWS_TABLEVIEW_TYPE, GwsTableView))
#define GWS_TABLEVIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GWS_TABLEVIEW_TYPE, GwsTableViewClass))
#define GWS_IS_TABLEVIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GWS_TABLEVIEW_TYPE))
#define GWS_IS_TABLEVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GWS_TABLEVIEW_TYPE))

typedef struct _GwsTableView       GwsTableView;
typedef struct _GwsTableViewClass  GwsTableViewClass;

struct _GwsTableView
{
    GtkTable table;
    GwyDataView *dataview;
    GwyContainer *container;
    GwyDataField *dfield;

    GtkWidget *xlabel;
    GtkWidget *ylabel;
    GtkObject *xadj;
    GtkObject *yadj;
    GtkObject *speedadj;

    gdouble xreq;
    gdouble yreq;
    gdouble xrange;
    gdouble yrange;
    gdouble speed;

    gint closing;

    gpointer md;
};

struct _GwsTableViewClass
{
  GtkTableClass parent_class;

  void (* tableview) (GwsTableView *tableview);
};

GType          gws_tableview_get_type        (void);
GtkWidget*     gws_tableview_new             (gpointer md);

void  gws_tableview_set_values(GwsTableView *tableview, gdouble xrange, gdouble yrange, gdouble xreq, gdouble yreq, gdouble speed);


G_END_DECLS

#endif /* __TABLEVIEW_H__ */
